manifest_schema = {
    "type": "object",
    "properties": {
        "sdk_version": {"type": "string"},
        "plugin_version": {"type": "string"},
        "name": {"type": "string"},
        "description": {"type": "string"},
        "secrets": {"type": "array", "items": {"type": "string"}},
        "origins": {"$ref": "#/$defs/origins"},
        "url_permissions": {"$ref": "#/$defs/url_permissions"},
        "components": {
            "type": "object",
            "properties": {
                "commands": {"$ref": "#/$defs/commands"},
                "protocols": {"$ref": "#/$defs/component"},
                "handlers": {"$ref": "#/$defs/component"},
                "content": {"$ref": "#/$defs/component"},
                "effects": {"$ref": "#/$defs/component"},
                "views": {"$ref": "#/$defs/component"},
                "applications": {"$ref": "#/$defs/applications"},
                "questionnaires": {"$ref": "#/$defs/questionnaires"},
            },
            "additionalProperties": False,
            "minProperties": 1,
        },
        "tags": {
            "type": "object",
            "properties": {
                "patient_sourcing_and_intake": {
                    "type": "array",
                    "items": {"enum": ["symptom_triage", "coverage_capture"]},
                },
                "interaction_modes_and_utilization": {
                    "type": "array",
                    "items": {"enum": ["supply_policies", "demand_policies", "auto_followup"]},
                },
                "diagnostic_range_and_inputs": {"type": "array", "items": {"enum": []}},
                "pricing_and_payments": {"type": "array", "items": {"enum": []}},
                "care_team_composition": {"type": "array", "items": {"enum": []}},
                "interventions_and_safety": {"type": "array", "items": {"enum": []}},
                "content": {"type": "array", "items": {"enum": ["patient_intake"]}},
            },
            "additionalProperties": False,
        },
        "references": {"type": "array", "items": {"type": "string"}},
        "license": {"type": "string"},
        "diagram": {"type": ["boolean", "string"]},
        "readme": {"type": ["boolean", "string"]},
    },
    "required": [
        "sdk_version",
        "plugin_version",
        "name",
        "description",
        "components",
        "tags",
        "license",
        "readme",
    ],
    "additionalProperties": False,
    "allOf": [{"not": {"required": ["url_permissions", "origins"]}}],
    "$defs": {
        "origins": {
            "type": "object",
            "properties": {
                "urls": {"type": "array", "items": {"type": "string"}},
                "scripts": {"type": "array", "items": {"type": "string"}},
            },
        },
        "url_permissions": {
            "type": "array",
            "items": {
                "type": "object",
                "additionalProperties": False,
                "properties": {
                    "url": {"type": "string"},
                    "permissions": {
                        "type": "array",
                        "items": {
                            "type": "string",
                            "enum": [
                                "SCRIPTS",
                                "ALLOW_SAME_ORIGIN",
                                "MICROPHONE",
                                "CAMERA",
                            ],
                        },
                    },
                },
                "required": ["url", "permissions"],
            },
        },
        "component": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "class": {"type": "string"},
                    "description": {"type": "string"},
                    "meta": {"type": "object"},
                    "data_access": {
                        "type": "object",
                        "properties": {
                            "event": {"type": "string"},
                            "read": {"type": "array", "items": {"type": "string"}},
                            "write": {"type": "array", "items": {"type": "string"}},
                        },
                        "additionalProperties": False,
                    },
                },
                "required": ["class", "description"],
                "additionalProperties": False,
            },
        },
        "applications": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "class": {"type": "string"},
                    "name": {"type": "string", "maxLength": 32},
                    "description": {"type": "string", "maxLength": 256},
                    "icon": {"type": "string"},
                    "scope": {
                        "type": "string",
                        "enum": [
                            "patient_specific",
                            "global",
                            "provider_menu_item",
                            "portal_menu_item",
                            "provider_companion",
                            "full_chart",
                        ],
                    },
                    "menu_position": {
                        "type": "string",
                        "enum": ["top", "bottom"],
                    },
                    "menu_order": {"type": "integer"},
                    "show_in_panel": {"type": "boolean"},
                    "panel_priority": {"type": "integer"},
                },
                "required": ["class", "icon", "scope", "name", "description"],
                "additionalProperties": False,
            },
        },
        "questionnaires": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "template": {"type": "string"},
                },
                "required": ["template"],
                "additionalProperties": False,
            },
        },
        "commands": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "name": {"type": "string"},
                    "label": {"type": "string"},
                    "schema_key": {"type": "string"},
                    "section": {
                        "type": "string",
                        "enum": [
                            "subjective",
                            "objective",
                            "assessment",
                            "plan",
                            "procedures",
                            "history",
                            "internal",
                        ],
                    },
                },
                "required": ["name", "schema_key"],
                "additionalProperties": False,
            },
        },
    },
}



import uuid

import factory

from canvas_sdk.v1.data import IntegrationTask, IntegrationTaskReview
from canvas_sdk.v1.data.integration_task import IntegrationTaskChannel, IntegrationTaskStatus


class IntegrationTaskFactory(factory.django.DjangoModelFactory[IntegrationTask]):
    """Factory for creating IntegrationTask."""

    class Meta:
        model = IntegrationTask

    id = factory.LazyFunction(uuid.uuid4)
    status = IntegrationTaskStatus.UNREAD
    type = factory.Faker("word")
    title = factory.Faker("sentence", nb_words=4)
    channel = IntegrationTaskChannel.FAX
    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")
    service_provider = factory.SubFactory("canvas_sdk.test_utils.factories.ServiceProviderFactory")


class IntegrationTaskReviewFactory(factory.django.DjangoModelFactory[IntegrationTaskReview]):
    """Factory for creating IntegrationTaskReview."""

    class Meta:
        model = IntegrationTaskReview

    id = factory.LazyFunction(uuid.uuid4)
    task = factory.SubFactory(IntegrationTaskFactory)
    template_name = factory.Faker("sentence", nb_words=3)
    document_key = factory.Faker("uuid4")
    reviewer = factory.SubFactory("canvas_sdk.test_utils.factories.StaffFactory")
    team_reviewer = None
    junked = False



from canvas_sdk.commands.base import _BaseCommand


class HistoryOfPresentIllnessCommand(_BaseCommand):
    """A class for managing a HPI command within a specific note."""

    class Meta:
        key = "hpi"

    narrative: str | None = None


__exports__ = ("HistoryOfPresentIllnessCommand",)



from typing import Any

from pydantic_core import InitErrorDetails

from canvas_sdk.commands.base import _BaseCommand


class CustomCommand(_BaseCommand):
    """A class for managing a custom command within a specific note.

    This class can be extended to create custom commands with predefined schema_key
    or passing schema_key directly to the constructor.
    """

    class Meta:
        key = "customCommand"
        schema_key: str | None = None

    _schema_key: str | None = None
    content: str | None = None
    print_content: str | None = None

    def __init_subclass__(cls, **kwargs: Any) -> None:
        """Ensure Meta.key is always 'customCommand' for all CustomCommand subclasses."""
        # Always enforce key = "customCommand" for all subclasses
        cls.Meta.key = "customCommand"

        # Call parent __init_subclass__ to perform validation
        super().__init_subclass__(**kwargs)

    def __init__(self, **data: Any) -> None:
        """Initialize with schema_key from Meta if not provided."""
        meta_schema_key = getattr(self.__class__.Meta, "schema_key", None)

        # Extract schema_key before init if provided
        instance_schema_key = data.pop("schema_key", None)

        # If Meta defines schema_key, use it and don't allow override
        if meta_schema_key is not None and instance_schema_key is not None:
            raise AttributeError(
                f"Cannot set schema_key on {self.__class__.__name__} instance. "
                f"schema_key is already defined in Meta as '{meta_schema_key}'."
            )

        super().__init__(**data)

        # Set _schema_key after init to ensure it's tracked
        if meta_schema_key is None and instance_schema_key is not None:
            self._schema_key = instance_schema_key

    @property
    def schema_key(self) -> str | None:
        """Get schema_key from Meta if defined, otherwise from instance attribute."""
        meta_schema_key = getattr(self.__class__.Meta, "schema_key", None)
        return meta_schema_key if meta_schema_key is not None else self._schema_key

    @property
    def values(self) -> dict:
        """Get values for the custom command."""
        return {**super().values, "schema_key": self.schema_key}

    def _get_error_details(self, method: str) -> list[InitErrorDetails]:
        """Get error details for the custom command."""
        errors = super()._get_error_details(method)

        if method == "originate":
            if not self.content:
                errors.append(
                    self._create_error_detail(
                        "value",
                        "Content must be provided for a custom command.",
                        self.content,
                    )
                )
            if not self.schema_key:
                errors.append(
                    self._create_error_detail(
                        "value",
                        "Schema key must be provided for a custom command.",
                        self.schema_key,
                    )
                )
        return errors


__exports__ = ("CustomCommand",)



from __future__ import annotations

from base64 import b64decode
from secrets import compare_digest
from typing import TYPE_CHECKING, Any, Protocol

from logger import log

from .exceptions import (
    AuthenticationSchemeError,
    InvalidCredentialsError,
    InvalidCredentialsFormatError,
    NoAuthorizationHeaderError,
)

if TYPE_CHECKING:
    from .api import Request


class Credentials:
    """
    Credentials base class.

    Performs no parsing of the request Authorization header. This class can be used as a base class
    for defining custom credentials classes, or can be specified by a developer as the credentials
    class if they wish to just access the request directly in their authentication method.
    """

    def __init__(self, request: Request) -> None:
        pass


class BasicCredentials(Credentials):
    """
    Basic authentication credentials class.

    Parses and decodes the username and password from the request Authorization header and saves
    them as attributes.
    """

    def __init__(self, request: Request) -> None:
        super().__init__(request)

        authorization = request.headers.get("Authorization")
        if not authorization:
            raise NoAuthorizationHeaderError

        scheme, delimiter, value = authorization.partition(" ")
        if delimiter != " ":
            raise AuthenticationSchemeError
        if scheme.lower() != "basic":
            raise AuthenticationSchemeError

        try:
            value = b64decode(value.encode()).decode()
        except Exception as exception:
            raise InvalidCredentialsFormatError from exception

        username, delimiter, password = value.partition(":")
        if delimiter != ":":
            raise InvalidCredentialsFormatError
        if not username or not password:
            raise InvalidCredentialsFormatError

        self.username = username
        self.password = password


class BearerCredentials(Credentials):
    """
    Bearer authentication credentials class.

    Parses the token from the request Authorization header and saves it as an attribute.
    """

    def __init__(self, request: Request) -> None:
        super().__init__(request)

        authorization = request.headers.get("Authorization")
        if not authorization:
            raise NoAuthorizationHeaderError

        scheme, delimiter, token = authorization.partition(" ")
        if delimiter != " ":
            raise AuthenticationSchemeError
        if scheme.lower() != "bearer":
            raise AuthenticationSchemeError
        if not token:
            raise InvalidCredentialsFormatError

        self.token = token


class APIKeyCredentials(Credentials):
    """
    API Key credentials class.

    Obtains the API key from the request Authorization header and saves it as an attribute.

    The default header name is "Authorization", but can be changed by overriding the HEADER_NAME
    class variable in subclasses.
    """

    HEADER_NAME = "Authorization"

    def __init__(self, request: Request) -> None:
        super().__init__(request)

        if self.HEADER_NAME not in request.headers:
            raise NoAuthorizationHeaderError

        key = request.headers.get(self.HEADER_NAME)
        if not key:
            raise InvalidCredentialsFormatError

        self.key = key


class SessionCredentials(Credentials):
    """
    Session credentials class.

    Looks for headers set by Canvas with information about the logged in user based on browser
    cookies and session information in the Canvas database. These headers are removed if received by
    the client and intentionally set by Canvas only if there is a valid session from an active user.

    One must look at the type of user in order to understand what the id refers to – a patient or a
    staff member.

    If a SimpleAPI handler is protected by SessionCredentials, the non-presence of these headers
    results in an unauthorized response. Users of SessionCredentials will likely wish to further
    restrict access based on the type of user (patient or staff) and perhaps based on the roles
    associated with the user. This logic is the concern of the implementer of the handler's
    authenticate method.
    """

    def __init__(self, request: Request) -> None:
        super().__init__(request)

        if (
            "canvas-logged-in-user-type" not in request.headers
            or "canvas-logged-in-user-id" not in request.headers
        ):
            raise InvalidCredentialsError

        self.logged_in_user = {
            "id": request.headers.get("canvas-logged-in-user-id"),
            "type": request.headers.get("canvas-logged-in-user-type"),
        }


class AuthSchemeMixin(Protocol):
    """Protocol for authentication scheme mixins."""

    secrets: dict[str, str]

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)

    def authenticate(self, credentials: Credentials) -> bool:
        """Authenticate the request."""
        ...


class BasicAuthMixin(AuthSchemeMixin):
    """
    Basic authentication scheme mixin.

    Provides an implementation of the authenticate method for Basic authentication.
    """

    USERNAME_SECRET_NAME = "simpleapi-basic-username"
    PASSWORD_SECRET_NAME = "simpleapi-basic-password"

    def authenticate(self, credentials: BasicCredentials) -> bool:  # type: ignore[override]
        """Authenticate the request."""
        try:
            username = self.secrets[self.USERNAME_SECRET_NAME]
            password = self.secrets[self.PASSWORD_SECRET_NAME]
        except KeyError as error:
            log.error(
                "SimpleAPI secrets for Basic authentication are not set; please set values for "
                f"{self.USERNAME_SECRET_NAME} and {self.PASSWORD_SECRET_NAME}"
            )
            raise InvalidCredentialsError from error

        if not (
            compare_digest(credentials.username.encode(), username.encode())
            and compare_digest(credentials.password.encode(), password.encode())
        ):
            raise InvalidCredentialsError

        return True


class APIKeyAuthMixin(AuthSchemeMixin):
    """
    API Key authentication scheme mixin.

    Provides an implementation of the authenticate method for API Key authentication.
    """

    API_KEY_SECRET_NAME = "simpleapi-api-key"

    def authenticate(self, credentials: APIKeyCredentials) -> bool:  # type: ignore[override]
        """Authenticate the request."""
        try:
            api_key = self.secrets[self.API_KEY_SECRET_NAME]
        except KeyError as error:
            log.error(
                f"SimpleAPI secret for API Key authentication is not set; please set a value for "
                f"{self.API_KEY_SECRET_NAME}"
            )
            raise InvalidCredentialsError from error

        if not compare_digest(credentials.key.encode(), api_key.encode()):
            raise InvalidCredentialsError

        return True


class StaffSessionAuthMixin(AuthSchemeMixin):
    """
    Staff Session authentication scheme mixin.

    Provides an implementation of the authenticate method that ensures only logged in staff members
    can access the urls provided by this handler. It only cares that they are a staff, with no
    regard to roles or any other permissioning schemes.
    """

    def authenticate(self, credentials: SessionCredentials) -> bool:  # type: ignore[override]
        """Authenticate the request."""
        if credentials.logged_in_user["type"] != "Staff":
            raise InvalidCredentialsError
        return True


class PatientSessionAuthMixin(AuthSchemeMixin):
    """
    Patient Session authentication scheme mixin.

    Provides an implementation of the authenticate method that ensures only logged in patients can
    access the urls provided by this handler. It only cares that they are a patient, with no regard
    any other information.
    """

    def authenticate(self, credentials: SessionCredentials) -> bool:  # type: ignore[override]
        """Authenticate the request."""
        if credentials.logged_in_user["type"] != "Patient":
            raise InvalidCredentialsError
        return True


__exports__ = (
    "Credentials",
    "BasicCredentials",
    "BearerCredentials",
    "APIKeyCredentials",
    "AuthSchemeMixin",
    "BasicAuthMixin",
    "APIKeyAuthMixin",
    "SessionCredentials",
    "StaffSessionAuthMixin",
    "PatientSessionAuthMixin",
)



from django.db import models


class Application(models.Model):
    """Plugin Application."""

    class Meta:
        db_table = "canvas_sdk_data_plugin_io_application_001"

    identifier = models.CharField(max_length=512, primary_key=True)
    name = models.TextField(null=False, max_length=32)
    description = models.TextField(null=False, max_length=256)


__exports__ = ("Application",)



from django.db import models

from canvas_sdk.v1.data.base import IdentifiableModel


class Facility(IdentifiableModel):
    """Facility model representing a healthcare facility."""

    class Meta:
        db_table = "canvas_sdk_data_api_facility_001"

    name = models.CharField(max_length=255)
    npi_number = models.CharField(verbose_name="NPI number", max_length=10, blank=True)
    phone_number = models.CharField(max_length=10, blank=True, default="")
    fax_number = models.CharField(max_length=10, blank=True, default="")
    active = models.BooleanField(default=True)
    line1 = models.CharField(max_length=255, default="", blank=True)
    line2 = models.CharField(max_length=255, default="", blank=True)
    city = models.CharField(max_length=255)
    district = models.CharField(max_length=255, blank=True, default="")
    state_code = models.CharField(max_length=2)
    postal_code = models.CharField(max_length=255)

    def __str__(self) -> str:
        return self.name


__exports__ = ("Facility",)



from canvas_sdk.clients.llms.structures.settings.llm_settings import LlmSettings
from canvas_sdk.clients.llms.structures.settings.llm_settings_anthropic import LlmSettingsAnthropic
from canvas_sdk.clients.llms.structures.settings.llm_settings_gemini import LlmSettingsGemini
from canvas_sdk.clients.llms.structures.settings.llm_settings_gpt4 import LlmSettingsGpt4
from canvas_sdk.clients.llms.structures.settings.llm_settings_gpt5 import LlmSettingsGpt5

__all__ = __exports__ = (
    "LlmSettings",
    "LlmSettingsAnthropic",
    "LlmSettingsGemini",
    "LlmSettingsGpt4",
    "LlmSettingsGpt5",
)



from abc import ABC, abstractmethod
from typing import Self


class Structure(ABC):
    """Abstract base class for data structures in the Extend AI client.

    All data structures that represent API entities must inherit from this class
    and implement methods for serialization and deserialization.
    """

    @classmethod
    @abstractmethod
    def from_dict(cls, data: dict) -> Self:
        """Create an instance from a dictionary representation.

        Args:
            data: A dictionary containing the data for this structure.

        Returns:
            An instance of the structure.
        """
        ...

    @abstractmethod
    def to_dict(self) -> dict:
        """Convert this instance to a dictionary representation.

        Returns:
            A dictionary representation of this structure.
        """
        ...


__exports__ = ()



import datetime
import json

import factory
from django.utils import timezone
from factory.fuzzy import FuzzyDate

from canvas_sdk.v1.data import (
    ImagingOrder,
    ImagingReport,
    ImagingReportTemplate,
    ImagingReportTemplateField,
    ImagingReportTemplateFieldOption,
    ImagingReview,
)
from canvas_sdk.v1.data.common import (
    DocumentReviewMode,
    OrderStatus,
    ReviewPatientCommunicationMethod,
    ReviewStatus,
)
from canvas_sdk.v1.data.imaging import ImagingReport as ImagingReportModel


class ImagingOrderFactory(factory.django.DjangoModelFactory[ImagingOrder]):
    """Factory for creating an ImagingOrder."""

    class Meta:
        model = ImagingOrder

    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")
    note = factory.SubFactory("canvas_sdk.test_utils.factories.NoteFactory")
    imaging = factory.Faker("text", max_nb_chars=1024)
    imaging_center = None  # Optional field
    note_to_radiologist = factory.Faker("text", max_nb_chars=1024)
    internal_comment = factory.Faker("text", max_nb_chars=1024)
    status = OrderStatus.REQUESTED
    date_time_ordered = factory.LazyFunction(lambda: timezone.now())
    priority = factory.Faker("random_element", elements=["routine", "urgent", "stat"])
    ordering_provider = factory.SubFactory("canvas_sdk.test_utils.factories.StaffFactory")
    delegated = False
    task_ids = factory.LazyFunction(lambda: json.dumps([]))


class ImagingReviewFactory(factory.django.DjangoModelFactory[ImagingReview]):
    """Factory for creating an ImagingReview."""

    class Meta:
        model = ImagingReview

    patient_communication_method = ReviewPatientCommunicationMethod.DELEGATED_CALL_CAN_LEAVE_MESSAGE
    internal_comment = factory.Faker("text", max_nb_chars=2048)
    message_to_patient = factory.Faker("text", max_nb_chars=2048)
    is_released_to_patient = False
    status = ReviewStatus.STATUS_REVIEWING
    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")


class ImagingReportFactory(factory.django.DjangoModelFactory[ImagingReport]):
    """Factory for creating an ImagingReport."""

    class Meta:
        model = ImagingReport

    review_mode = DocumentReviewMode.REVIEW_REQUIRED
    junked = False
    requires_signature = False
    assigned_date = factory.LazyFunction(lambda: timezone.now())
    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")
    order = None  # Optional field
    source = ImagingReportModel.ImagingReportSource.DIRECTLY_REPORT
    name = factory.Faker("text", max_nb_chars=255)
    result_date = FuzzyDate(
        start_date=datetime.date.today() - datetime.timedelta(days=365),
        end_date=datetime.date.today(),
    )
    original_date = FuzzyDate(
        start_date=datetime.date.today() - datetime.timedelta(days=365),
        end_date=datetime.date.today(),
    )
    review = None  # Optional field


class ImagingReportTemplateFactory(factory.django.DjangoModelFactory[ImagingReportTemplate]):
    """Factory for creating an ImagingReportTemplate."""

    class Meta:
        model = ImagingReportTemplate

    name = factory.Faker("text", max_nb_chars=250)
    long_name = factory.Faker("text", max_nb_chars=500)
    code = factory.Faker("bothify", text="?????")
    code_system = factory.Faker("random_element", elements=["CPT", "LOINC", "SNOMED"])
    search_keywords = factory.Faker("text", max_nb_chars=200)
    active = True
    custom = False
    rank = factory.Faker("random_int", min=1, max=100)


class ImagingReportTemplateFieldFactory(
    factory.django.DjangoModelFactory[ImagingReportTemplateField]
):
    """Factory for creating an ImagingReportTemplateField."""

    class Meta:
        model = ImagingReportTemplateField

    report_template = factory.SubFactory(ImagingReportTemplateFactory)
    sequence = factory.Sequence(lambda n: n + 1)
    code = factory.Faker("bothify", text="?????")
    code_system = factory.Faker("random_element", elements=["LOINC", "SNOMED"])
    label = factory.Faker("text", max_nb_chars=100)
    units = factory.Faker("random_element", elements=["mg", "mL", "cm", None])
    type = factory.Faker("random_element", elements=["text", "float", "select", "integer"])
    required = factory.Faker("boolean")


class ImagingReportTemplateFieldOptionFactory(
    factory.django.DjangoModelFactory[ImagingReportTemplateFieldOption]
):
    """Factory for creating an ImagingReportTemplateFieldOption."""

    class Meta:
        model = ImagingReportTemplateFieldOption

    field = factory.SubFactory(ImagingReportTemplateFieldFactory)
    label = factory.Faker("text", max_nb_chars=100)
    key = factory.Faker("slug")



from typing import Self
from uuid import UUID

from django.db import models

from canvas_sdk.v1.data.base import (
    BaseQuerySet,
    ForPatientQuerySetMixin,
    IdentifiableModel,
    TimestampedModel,
)


class IntegrationTaskStatus(models.TextChoices):
    """Status choices for IntegrationTask."""

    UNREAD = "UNR", "Unread"
    UNREAD_ERROR = "UER", "Unread Error"
    READ = "REA", "Read"
    ERROR = "ERR", "Error"
    PROCESSED = "PRO", "Processed"
    REVIEWED = "REV", "Reviewed"
    JUNK = "JUN", "Junk"


class IntegrationTaskChannel(models.TextChoices):
    """Channel choices for IntegrationTask."""

    FAX = "fax", "Fax"
    DOCUMENT_UPLOAD = "document_upload", "Document Upload"
    FROM_INTEGRATION_ENGINE = "from_integration_engine", "From Integration Engine"
    FROM_PATIENT_PORTAL = "from_patient_portal", "From Patient Portal"


class IntegrationTaskQuerySet(ForPatientQuerySetMixin, BaseQuerySet):
    """QuerySet for IntegrationTask with custom filter methods."""

    # Status filters
    def unread(self) -> Self:
        """Filter to unread tasks."""
        return self.filter(status=IntegrationTaskStatus.UNREAD)

    def pending_review(self) -> Self:
        """Filter to tasks pending review (UNREAD or READ)."""
        return self.filter(status__in=[IntegrationTaskStatus.UNREAD, IntegrationTaskStatus.READ])

    def processed(self) -> Self:
        """Filter to processed tasks (PROCESSED or REVIEWED)."""
        return self.filter(
            status__in=[IntegrationTaskStatus.PROCESSED, IntegrationTaskStatus.REVIEWED]
        )

    def with_errors(self) -> Self:
        """Filter to tasks with errors (ERROR or UNREAD_ERROR)."""
        return self.filter(
            status__in=[IntegrationTaskStatus.ERROR, IntegrationTaskStatus.UNREAD_ERROR]
        )

    def junked(self) -> Self:
        """Filter to junked tasks."""
        return self.filter(status=IntegrationTaskStatus.JUNK)

    def not_junked(self) -> Self:
        """Filter to non-junked tasks."""
        return self.exclude(status=IntegrationTaskStatus.JUNK)

    # Channel filters
    def faxes(self) -> Self:
        """Filter to fax tasks."""
        return self.filter(channel=IntegrationTaskChannel.FAX)

    def uploads(self) -> Self:
        """Filter to document upload tasks."""
        return self.filter(channel=IntegrationTaskChannel.DOCUMENT_UPLOAD)

    def from_integration_engine(self) -> Self:
        """Filter to tasks from integration engine."""
        return self.filter(channel=IntegrationTaskChannel.FROM_INTEGRATION_ENGINE)

    def from_patient_portal(self) -> Self:
        """Filter to tasks from patient portal."""
        return self.filter(channel=IntegrationTaskChannel.FROM_PATIENT_PORTAL)


class IntegrationTaskReviewQuerySet(BaseQuerySet):
    """QuerySet for IntegrationTaskReview with custom filter methods."""

    def for_task(self, task_id: str) -> Self:
        """Filter reviews by task ID."""
        return self.filter(task__id=task_id)

    def junked(self) -> Self:
        """Filter to junked reviews."""
        return self.filter(junked=True)

    def not_junked(self) -> Self:
        """Filter to non-junked reviews."""
        return self.filter(junked=False)

    def active(self) -> Self:
        """Filter to active (non-junked) reviews."""
        return self.not_junked()

    def by_reviewer(self, reviewer_id: str | UUID) -> Self:
        """Filter reviews by reviewer ID."""
        return self.filter(reviewer__id=reviewer_id)

    def by_team(self, team_id: str) -> Self:
        """Filter reviews by team reviewer ID."""
        return self.filter(team_reviewer__id=team_id)


class IntegrationTask(TimestampedModel, IdentifiableModel):
    """IntegrationTask - represents incoming documents that need processing.

    This includes faxes, document uploads, lab results, and patient portal submissions.
    """

    class Meta:
        db_table = "canvas_sdk_data_data_integration_integrationtask_001"

    objects = models.Manager.from_queryset(IntegrationTaskQuerySet)()

    status = models.CharField(
        max_length=3,
        choices=IntegrationTaskStatus.choices,
        default=IntegrationTaskStatus.UNREAD,
        db_index=True,
    )
    type = models.CharField(max_length=125, blank=True, default="")
    title = models.CharField(max_length=256, blank=True, default="")
    channel = models.CharField(max_length=50, choices=IntegrationTaskChannel.choices, db_index=True)
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.DO_NOTHING, null=True, related_name="integration_tasks"
    )
    service_provider = models.ForeignKey(
        "v1.ServiceProvider",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="integration_tasks",
    )

    @property
    def is_fax(self) -> bool:
        """Check if this is a fax task."""
        return self.channel == IntegrationTaskChannel.FAX

    @property
    def is_pending(self) -> bool:
        """Check if this task is pending review."""
        return self.status in [IntegrationTaskStatus.UNREAD, IntegrationTaskStatus.READ]

    @property
    def is_processed(self) -> bool:
        """Check if this task has been processed."""
        return self.status in [IntegrationTaskStatus.PROCESSED, IntegrationTaskStatus.REVIEWED]

    @property
    def has_error(self) -> bool:
        """Check if this task has an error."""
        return self.status in [IntegrationTaskStatus.ERROR, IntegrationTaskStatus.UNREAD_ERROR]

    @property
    def is_junked(self) -> bool:
        """Check if this task is junked."""
        return self.status == IntegrationTaskStatus.JUNK


class IntegrationTaskReview(TimestampedModel, IdentifiableModel):
    """IntegrationTaskReview - represents a review assignment for an integration task.

    Reviews track who is responsible for processing a document and its current state.
    """

    class Meta:
        db_table = "canvas_sdk_data_data_integration_integrationtaskreview_001"

    objects = models.Manager.from_queryset(IntegrationTaskReviewQuerySet)()

    task = models.ForeignKey(
        IntegrationTask,
        on_delete=models.DO_NOTHING,
        related_name="reviews",
    )
    template_name = models.CharField(max_length=1000, null=True)
    document_key = models.CharField(max_length=250, null=True, blank=True)
    reviewer = models.ForeignKey(
        "v1.Staff",
        on_delete=models.DO_NOTHING,
        related_name="integration_task_reviews",
        null=True,
    )
    team_reviewer = models.ForeignKey(
        "v1.Team",
        on_delete=models.DO_NOTHING,
        related_name="integration_task_team_reviews",
        null=True,
        blank=True,
    )
    junked = models.BooleanField(default=False)

    @property
    def is_active(self) -> bool:
        """Check if this review is active (not junked)."""
        return not self.junked


__exports__ = (
    "IntegrationTask",
    "IntegrationTaskStatus",
    "IntegrationTaskChannel",
    "IntegrationTaskQuerySet",
    "IntegrationTaskReview",
    "IntegrationTaskReviewQuerySet",
)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from canvas_generated.messages import events_pb2 as canvas__generated_dot_messages_dot_events__pb2
from canvas_generated.messages import plugins_pb2 as canvas__generated_dot_messages_dot_plugins__pb2

GRPC_GENERATED_VERSION = '1.78.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in canvas_generated/services/plugin_runner_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class PluginRunnerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.HandleEvent = channel.unary_stream(
                '/canvas.PluginRunner/HandleEvent',
                request_serializer=canvas__generated_dot_messages_dot_events__pb2.Event.SerializeToString,
                response_deserializer=canvas__generated_dot_messages_dot_events__pb2.EventResponse.FromString,
                _registered_method=True)
        self.ReloadPlugins = channel.unary_stream(
                '/canvas.PluginRunner/ReloadPlugins',
                request_serializer=canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginsRequest.SerializeToString,
                response_deserializer=canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginsResponse.FromString,
                _registered_method=True)
        self.ReloadPlugin = channel.unary_stream(
                '/canvas.PluginRunner/ReloadPlugin',
                request_serializer=canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginRequest.SerializeToString,
                response_deserializer=canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginResponse.FromString,
                _registered_method=True)
        self.UnloadPlugin = channel.unary_stream(
                '/canvas.PluginRunner/UnloadPlugin',
                request_serializer=canvas__generated_dot_messages_dot_plugins__pb2.UnloadPluginRequest.SerializeToString,
                response_deserializer=canvas__generated_dot_messages_dot_plugins__pb2.UnloadPluginResponse.FromString,
                _registered_method=True)


class PluginRunnerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def HandleEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReloadPlugins(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReloadPlugin(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnloadPlugin(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PluginRunnerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'HandleEvent': grpc.unary_stream_rpc_method_handler(
                    servicer.HandleEvent,
                    request_deserializer=canvas__generated_dot_messages_dot_events__pb2.Event.FromString,
                    response_serializer=canvas__generated_dot_messages_dot_events__pb2.EventResponse.SerializeToString,
            ),
            'ReloadPlugins': grpc.unary_stream_rpc_method_handler(
                    servicer.ReloadPlugins,
                    request_deserializer=canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginsRequest.FromString,
                    response_serializer=canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginsResponse.SerializeToString,
            ),
            'ReloadPlugin': grpc.unary_stream_rpc_method_handler(
                    servicer.ReloadPlugin,
                    request_deserializer=canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginRequest.FromString,
                    response_serializer=canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginResponse.SerializeToString,
            ),
            'UnloadPlugin': grpc.unary_stream_rpc_method_handler(
                    servicer.UnloadPlugin,
                    request_deserializer=canvas__generated_dot_messages_dot_plugins__pb2.UnloadPluginRequest.FromString,
                    response_serializer=canvas__generated_dot_messages_dot_plugins__pb2.UnloadPluginResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'canvas.PluginRunner', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('canvas.PluginRunner', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class PluginRunner(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def HandleEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/canvas.PluginRunner/HandleEvent',
            canvas__generated_dot_messages_dot_events__pb2.Event.SerializeToString,
            canvas__generated_dot_messages_dot_events__pb2.EventResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ReloadPlugins(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/canvas.PluginRunner/ReloadPlugins',
            canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginsRequest.SerializeToString,
            canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ReloadPlugin(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/canvas.PluginRunner/ReloadPlugin',
            canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginRequest.SerializeToString,
            canvas__generated_dot_messages_dot_plugins__pb2.ReloadPluginResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def UnloadPlugin(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/canvas.PluginRunner/UnloadPlugin',
            canvas__generated_dot_messages_dot_plugins__pb2.UnloadPluginRequest.SerializeToString,
            canvas__generated_dot_messages_dot_plugins__pb2.UnloadPluginResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



from dataclasses import dataclass


@dataclass(frozen=True)
class ParsedAttachment:
    """Represents an attachment from an inbound parsed email."""

    filename: str
    name: str
    type: str
    content_id: str

    @classmethod
    def from_dict(cls, data: dict) -> "ParsedAttachment":
        """Create ParsedAttachment instance from dictionary."""
        return cls(
            filename=data["filename"],
            name=data["name"],
            type=data["type"],
            content_id=data["content-id"],
        )

    def to_dict(self) -> dict:
        """Convert parsed attachment to dictionary format."""
        return {
            "filename": self.filename,
            "name": self.name,
            "type": self.type,
            "contentId": self.content_id,
        }


__exports__ = ()



import importlib.metadata
from pathlib import Path

import typer

from canvas_cli.apps import plugin
from canvas_cli.apps.emit import emit
from canvas_cli.apps.logs import logs as logs_command
from canvas_cli.apps.run_plugins import run_plugin, run_plugins
from canvas_cli.utils.context import context

APP_NAME = "canvas_cli"

# The main app
app = typer.Typer(no_args_is_help=True, rich_markup_mode=None, add_completion=False)

# Commands
app.command(short_help="Create a new plugin")(plugin.init)
app.command(short_help="Install a plugin into a Canvas instance")(plugin.install)
app.command(short_help="Uninstall a plugin from a Canvas instance")(plugin.uninstall)
app.command(short_help="Enable a plugin from a Canvas instance")(plugin.enable)
app.command(short_help="Disable a plugin from a Canvas instance")(plugin.disable)
app.command(short_help="List all plugins from a Canvas instance")(plugin.list)
app.command(short_help="Validate the Canvas Manifest json file")(plugin.validate_manifest)
app.command(
    short_help="Listen and print log streams or fetches historical logs from a Canvas instance."
)(logs_command)
app.command(
    short_help="Send an event fixture to your locally running plugin-runner process, and print any resultant effects."
)(emit)
app.command(short_help="Run the specified plugins for local development.")(run_plugins)
app.command(short_help="Run the specified plugin for local development.")(run_plugin)

# Config app
config_app = typer.Typer(help="Manage plugin secrets.", rich_markup_mode=None, add_completion=False)
app.add_typer(config_app, name="config")
config_app.command(name="list", short_help="List plugin secrets on a Canvas instance.")(
    plugin.list_secrets
)
config_app.command(name="set", short_help="Set plugin secrets on a Canvas instance.")(
    plugin.set_secrets
)

# Our current version
__version__ = importlib.metadata.version("canvas")


def version_callback(value: bool) -> None:
    """Method called when the `--version` flag is set. Prints the version and exits the CLI."""
    if value:
        print(f"{APP_NAME} Version: {__version__}")
        raise typer.Exit()


def get_app_dir() -> str:
    """Return the app dir, where the config file will be saved.
    This method is monkeypatched in conftest.py, for testing purposes.
    """
    return typer.get_app_dir(APP_NAME)


def get_or_create_config_file() -> Path:
    """Method called to get a Path to the existent JSON config file, or create one if it doesn't exist."""
    app_dir = get_app_dir()
    config_path: Path = Path(app_dir) / "config.json"
    if not config_path.is_file():
        Path(app_dir).mkdir(parents=True, exist_ok=True)
        with open(config_path, "w+") as file:
            file.write("{}")

    return config_path


@app.callback()
def main(
    version: bool | None = typer.Option(
        None, "--version", callback=version_callback, is_eager=True
    ),
) -> None:
    """Canvas swiss army knife CLI tool."""
    # Fetch the config file and load our context from it.
    config_file = get_or_create_config_file()

    context.load_from_file(config_file)


if __name__ == "__main__":
    app()



from canvas_sdk.effects.observation.base import (
    CodingData,
    Observation,
    ObservationComponentData,
)

__all__ = __exports__ = (
    "Observation",
    "CodingData",
    "ObservationComponentData",
)



from __future__ import annotations

from typing import NamedTuple


class Credentials(NamedTuple):
    """Credentials for S3 access.

    Attributes:
        key: AWS access key ID
        secret: AWS secret access key
        region: AWS region (e.g., 'us-east-1')
        bucket: S3 bucket name
    """

    key: str
    secret: str
    region: str
    bucket: str


__exports__ = ("Credentials",)



from abc import ABC, abstractmethod

from canvas_sdk.effects import Effect
from canvas_sdk.events import EventType
from canvas_sdk.handlers import BaseHandler
from canvas_sdk.handlers.action_button import ActionButton
from canvas_sdk.handlers.utils import normalize_effects


class Application(BaseHandler, ABC):
    """An embeddable application that can be registered to Canvas."""

    RESPONDS_TO = [
        EventType.Name(EventType.APPLICATION__ON_OPEN),
        EventType.Name(EventType.APPLICATION__ON_CONTEXT_CHANGE),
    ]

    def compute(self) -> list[Effect]:
        """Handle the application events."""
        if self.event.target.id != self.identifier:
            return []
        match self.event.type:
            case EventType.APPLICATION__ON_OPEN:
                return normalize_effects(self.on_open())
            case EventType.APPLICATION__ON_CONTEXT_CHANGE:
                return normalize_effects(self.on_context_change())
            case _:
                return []

    @abstractmethod
    def on_open(self) -> Effect | list[Effect]:
        """Handle the application open event."""
        ...

    def on_context_change(self) -> Effect | list[Effect] | None:
        """Handle the application context change event."""
        return None

    @property
    def identifier(self) -> str:
        """The application identifier."""
        return f"{self.__class__.__module__}:{self.__class__.__qualname__}"


class NoteApplication(ActionButton):
    """An Application that can be shown in a note."""

    NAME: str = ""
    IDENTIFIER: str = ""

    @property
    def BUTTON_TITLE(self) -> str:  # type: ignore[override]
        """Return NAME as the button title."""
        return self.NAME

    @property
    def BUTTON_KEY(self) -> str:  # type: ignore[override]
        """Return IDENTIFIER as the button key."""
        return self.IDENTIFIER

    @property
    def BUTTON_LOCATION(self) -> ActionButton.ButtonLocation:  # type: ignore[override]
        """Return the note body as the button location."""
        return ActionButton.ButtonLocation.NOTE_BODY

    @abstractmethod
    def handle(self) -> list[Effect]:
        """Method to handle button click."""
        raise NotImplementedError("Implement to handle button click")


__exports__ = (
    "Application",
    "NoteApplication",
)



from dataclasses import dataclass
from typing import Self

from canvas_sdk.clients.extend_ai.constants.base_processor import BaseProcessor
from canvas_sdk.clients.extend_ai.constants.processor_type import ProcessorType
from canvas_sdk.clients.extend_ai.structures.classification import Classification
from canvas_sdk.clients.extend_ai.structures.config.advanced_options_classification import (
    AdvancedOptionsClassification,
)
from canvas_sdk.clients.extend_ai.structures.config.config_base import ConfigBase
from canvas_sdk.clients.extend_ai.structures.config.parser import Parser


@dataclass(frozen=True)
class ConfigClassification(ConfigBase):
    """Configuration for a classification processor.

    Attributes:
        classifications: List of possible classification categories.
        base_processor: The base processor type to use (performance or light).
        classification_rule: Custom rules to guide the classification.
        advanced_options: Advanced configuration options for classification.
        parser: Parser configuration for document processing.
    """

    classifications: list[Classification]
    base_processor: BaseProcessor
    classification_rule: str
    advanced_options: AdvancedOptionsClassification
    parser: Parser

    @classmethod
    def processor_type(cls) -> ProcessorType:
        """Get the processor type for this configuration.

        Returns:
            ProcessorType.CLASSIFY for classification processors.
        """
        return ProcessorType.CLASSIFY

    @classmethod
    def from_dict(cls, data: dict) -> Self:
        """Create a ConfigClassification instance from a dictionary.

        Args:
            data: Dictionary containing classification configuration from the API.

        Returns:
            A new ConfigClassification instance with defaults for missing values.
        """
        return cls(
            classifications=[Classification.from_dict(item) for item in data["classifications"]],
            base_processor=BaseProcessor(data["baseProcessor"]),
            classification_rule=data.get("classificationRules") or "",
            advanced_options=AdvancedOptionsClassification.from_dict(data["advancedOptions"]),
            parser=Parser.from_dict(data.get("parser") or {}),
        )

    def to_dict(self) -> dict:
        """Convert this ConfigClassification to a dictionary.

        Returns:
            Dictionary representation suitable for API requests.
        """
        return {
            "type": ProcessorType.CLASSIFY.value,
            "classifications": [item.to_dict() for item in self.classifications],
            "baseProcessor": self.base_processor.value,
            "classificationRules": self.classification_rule,
            "advancedOptions": self.advanced_options.to_dict(),
            "parser": self.parser.to_dict(),
        }


__exports__ = ()



from datetime import date
from enum import Enum, IntEnum

from typing_extensions import TypedDict

from canvas_sdk.commands.base import _BaseCommand as BaseCommand


class AllergenType(IntEnum):
    """An enum representing the type of allergen."""

    ALLERGEN_GROUP = 1
    MEDICATION = 2
    INGREDIENT = 6


class Allergen(TypedDict):
    """A TypedDict representing an allergen."""

    concept_id: int
    concept_type: AllergenType


class AllergyCommand(BaseCommand):
    """A class for managing an Allergy command within a specific note."""

    class Meta:
        key = "allergy"

    class Severity(Enum):
        MILD = "mild"
        MODERATE = "moderate"
        SEVERE = "severe"

    allergy: Allergen | None = None
    severity: Severity | None = None
    narrative: str | None = None
    approximate_date: date | None = None


__exports__ = (
    "AllergenType",
    "Allergen",
    "AllergyCommand",
)



from typing import Any
from uuid import UUID

from canvas_sdk.effects.metadata import BaseMetadata
from canvas_sdk.v1.data import Claim


class _ClaimMetadata(BaseMetadata):
    """Effect to upsert a Claim Metadata record."""

    class Meta:
        effect_type = "CLAIM_METADATA"

    claim_id: UUID | str

    def _get_error_details(self, method: Any) -> list:
        errors = super()._get_error_details(method)

        if not Claim.objects.filter(id=self.claim_id).exists():
            errors.append(
                self._create_error_detail(
                    "claim_id",
                    f"Claim with id: {self.claim_id} does not exist.",
                    self.claim_id,
                )
            )

        return errors


__exports__ = ()



from http import HTTPStatus
from typing import NamedTuple

from canvas_sdk.clients.llms.structures.llm_tokens import LlmTokens


class LlmResponse(NamedTuple):
    """Response from an LLM API call.

    Attributes:
        code: HTTP status code of the response.
        response: Text content returned by the LLM.
        tokens: Token usage information for the request.
    """

    code: HTTPStatus
    response: str
    tokens: LlmTokens

    def to_dict(self) -> dict:
        """Convert the response to a dictionary representation.

        Returns:
            Dictionary containing the response data with serialized code, response text, and tokens.
        """
        return {
            "code": self.code.value,
            "response": self.response,
            "tokens": self.tokens.to_dict(),
        }


__exports__ = ("LlmResponse",)



from collections.abc import Iterator
from http import HTTPStatus
from typing import TypeVar

from requests import Response

from canvas_sdk.clients.sendgrid.constants.recipient_type import RecipientType
from canvas_sdk.clients.sendgrid.structures.email import Email
from canvas_sdk.clients.sendgrid.structures.event_webhook import EventWebhook
from canvas_sdk.clients.sendgrid.structures.event_webhook_record import EventWebhookRecord
from canvas_sdk.clients.sendgrid.structures.logged_email_criteria import LoggedEmailCriteria
from canvas_sdk.clients.sendgrid.structures.parse_setting import ParseSetting
from canvas_sdk.clients.sendgrid.structures.request_failed import RequestFailed
from canvas_sdk.clients.sendgrid.structures.sent_email import SentEmail
from canvas_sdk.clients.sendgrid.structures.sent_email_detail import SentEmailDetail
from canvas_sdk.clients.sendgrid.structures.settings import Settings
from canvas_sdk.clients.sendgrid.structures.structure import Structure
from canvas_sdk.utils import Http

Data = TypeVar("Data", bound=Structure)


class EmailClient:
    """Client for interacting with SendGrid API."""

    def __init__(self, settings: Settings):
        """Initialize EmailClient with API settings."""
        self.settings = settings
        self.http = Http("https://api.sendgrid.com")

    def _auth_header(self) -> dict:
        """Generate authorization header with API key."""
        return {
            "Authorization": f"Bearer {self.settings.key}",
        }

    @classmethod
    def _valid_content_bool(cls, request: Response, status: HTTPStatus) -> bool:
        """Validate response status and return True on success.

        Args:
            request: The HTTP response object
            status: Expected status code

        Returns:
            True if status code matches

        Raises:
            RequestFailed: If status code does not match expected
        """
        if request.status_code == status:
            return True
        raise RequestFailed(status_code=request.status_code, message=request.content.decode())

    @classmethod
    def _valid_content(
        cls,
        request: Response,
        statuses: list[HTTPStatus],
        returned: type[Data],
    ) -> Data:
        """Validate response status and parse single object.

        Args:
            request: The HTTP response object
            statuses: List of acceptable status codes
            returned: Structure class to parse response data into

        Returns:
            Parsed object of the specified type

        Raises:
            RequestFailed: If status code is not in acceptable list
        """
        if request.status_code in statuses:
            return returned.from_dict(request.json())
        raise RequestFailed(status_code=request.status_code, message=request.content.decode())

    @classmethod
    def _valid_content_list(
        cls,
        request: Response,
        status: HTTPStatus,
        key: str,
        returned: type[Data],
    ) -> Iterator[Data]:
        """Validate response status and yield parsed objects from list.

        Args:
            request: The HTTP response object
            status: Expected status code
            key: JSON key to extract list from response
            returned: Structure class to parse each item into

        Yields:
            Parsed objects of the specified type

        Raises:
            RequestFailed: If status code does not match expected
        """
        if request.status_code != status:
            raise RequestFailed(status_code=request.status_code, message=request.content.decode())
        for item in request.json()[key]:
            yield returned.from_dict(item)

    def simple_send(self, email: Email) -> bool:
        """Send an email using simplified interface.

        This method provides an easy way to send emails without complexity requirements.
        It uses the prepared_send method internally, which provides support for
        advanced and complex email-sending workflows.

        Note: SendGrid enforces the presence of at least one email in the `to` section.
        """
        per_recipient_type = {
            "to": RecipientType.TO,
            "cc": RecipientType.CC,
            "bcc": RecipientType.BCC,
        }
        data = {
            "from": email.sender.to_dict(),
            "reply_to_list": [r.to_dict() for r in email.reply_tos],
            "subject": email.subject,
            "content": [b.to_dict() for b in email.bodies],
            "send_at": email.send_at,
            "personalizations": [
                {
                    key: listed
                    for key, recipient_type in per_recipient_type.items()
                    if (
                        listed := [
                            r.address.to_dict()
                            for r in email.recipients
                            if r.type == recipient_type
                        ]
                    )
                }
            ],
        }
        if attachments := [a.to_dict() for a in email.attachments]:
            data["attachments"] = attachments
        return self.prepared_send(data)

    def prepared_send(self, data: dict) -> bool:
        """Send an email using prepared data dictionary.

        The data dictionary must follow SendGrid's schema:
        https://www.twilio.com/docs/sendgrid/api-reference/mail-send/mail-send#request-body
        """
        url = "/v3/mail/send"
        headers = {"Content-Type": "application/json"} | self._auth_header()
        request = self.http.post(url, json=data, headers=headers)
        return self._valid_content_bool(request, HTTPStatus.ACCEPTED)

    def logged_emails(self, criteria: LoggedEmailCriteria, up_to: int) -> Iterator[SentEmail]:
        """Retrieve logged emails matching search criteria."""
        url = "/v3/logs"
        headers = {"Content-Type": "application/json"} | self._auth_header()
        data = {
            "query": criteria.to_str(),
            "limit": up_to,
        }
        request = self.http.post(url, json=data, headers=headers)
        yield from self._valid_content_list(request, HTTPStatus.OK, "messages", SentEmail)

    def logged_email(self, message_id: str) -> SentEmailDetail:
        """Retrieve detailed information for a specific email by message ID."""
        url = f"/v3/logs/{message_id}"
        request = self.http.get(url, headers=self._auth_header())
        return self._valid_content(request, [HTTPStatus.OK], SentEmailDetail)

    # incoming emails:
    # - SendGrid UI  : https://app.sendgrid.com/settings/parse
    # - documentation: https://www.twilio.com/docs/sendgrid/for-developers/parsing-email/inbound-email
    def parser_setting_add(self, setting: ParseSetting) -> ParseSetting:
        """Add a new inbound parse webhook setting."""
        url = "/v3/user/webhooks/parse/settings"
        headers = {"Content-Type": "application/json"} | self._auth_header()
        request = self.http.post(url, json=setting.to_dict(), headers=headers)
        # discrepancy between the doc and the reality:
        # - 201 according to: https://www.twilio.com/docs/sendgrid/api-reference/settings-inbound-parse/create-a-parse-setting#responses
        # - 200 based on real usages
        return self._valid_content(request, [HTTPStatus.OK, HTTPStatus.CREATED], ParseSetting)

    def parser_setting_delete(self, hostname: str) -> bool:
        """Delete an inbound parse webhook setting by hostname."""
        url = f"/v3/user/webhooks/parse/settings/{hostname}"
        request = self.http.delete(url, headers=self._auth_header())
        return self._valid_content_bool(request, HTTPStatus.NO_CONTENT)

    def parser_setting_get(self, hostname: str) -> ParseSetting:
        """Retrieve an inbound parse webhook setting by hostname."""
        url = f"/v3/user/webhooks/parse/settings/{hostname}"
        request = self.http.get(url, headers=self._auth_header())
        return self._valid_content(request, [HTTPStatus.OK], ParseSetting)

    def parser_setting_list(self) -> Iterator[ParseSetting]:
        """List all inbound parse webhook settings."""
        url = "/v3/user/webhooks/parse/settings"
        request = self.http.get(url, headers=self._auth_header())
        yield from self._valid_content_list(request, HTTPStatus.OK, "result", ParseSetting)

    # sent email statuses:
    # - SendGrid UI  : https://app.sendgrid.com/settings/mail_settings/webhook_settings
    # - documentation: https://www.twilio.com/docs/sendgrid/for-developers/tracking-events/event
    def event_webhook_add(self, event: EventWebhook) -> EventWebhookRecord:
        """Add a new event webhook for email status notifications."""
        url = "/v3/user/webhooks/event/settings"
        headers = {"Content-Type": "application/json"} | self._auth_header()
        request = self.http.post(url, json=event.to_dict(), headers=headers)
        return self._valid_content(request, [HTTPStatus.CREATED], EventWebhookRecord)

    def event_webhook_sign(self, event_webhook_id: str, enabled: bool) -> str:
        """Enable or disable webhook signature verification and return public key."""
        url = f"/v3/user/webhooks/event/settings/signed/{event_webhook_id}"
        headers = {"Content-Type": "application/json"} | self._auth_header()
        request = self.http.patch(url, json={"enabled": enabled}, headers=headers)
        if request.status_code == HTTPStatus.OK:
            return request.json().get("public_key") or ""
        raise RequestFailed(status_code=request.status_code, message=request.content.decode())

    def event_webhook_delete(self, event_webhook_id: str) -> bool:
        """Delete an event webhook by ID."""
        url = f"/v3/user/webhooks/event/settings/{event_webhook_id}"
        request = self.http.delete(url, headers=self._auth_header())
        return self._valid_content_bool(request, HTTPStatus.NO_CONTENT)

    def event_webhook_get(self, event_webhook_id: str) -> EventWebhookRecord:
        """Retrieve an event webhook by ID."""
        url = f"/v3/user/webhooks/event/settings/{event_webhook_id}"
        request = self.http.get(url, headers=self._auth_header())
        return self._valid_content(request, [HTTPStatus.OK], EventWebhookRecord)

    def event_webhook_list(self) -> Iterator[EventWebhookRecord]:
        """List all event webhooks."""
        url = "/v3/user/webhooks/event/settings/all"
        request = self.http.get(url, headers=self._auth_header())
        yield from self._valid_content_list(request, HTTPStatus.OK, "webhooks", EventWebhookRecord)


__exports__ = ()



import json
from abc import ABC, abstractmethod
from decimal import Decimal
from enum import StrEnum
from typing import Any

from canvas_sdk.effects import Effect
from canvas_sdk.effects.payment_processor import (
    AddPaymentMethodResponse,
    CardTransaction,
    PaymentMethod,
    PaymentProcessorForm,
    PaymentProcessorMetadata,
    RemovePaymentMethodResponse,
)
from canvas_sdk.handlers.payment_processors.base import PaymentProcessor
from canvas_sdk.v1.data import Patient


class CardPaymentProcessor(PaymentProcessor, ABC):
    """Base Card Payment Processor Handler."""

    TYPE = PaymentProcessorMetadata.PaymentProcessorType.CARD

    class PaymentIntent(StrEnum):
        """Enum for payment actions."""

        ADD_CARD = "add_card"
        PAY = "pay"

    def _on_payment_processor_selected(self) -> list[Effect]:
        """Handle the event when a payment processor is selected."""
        if self.event.context.get("identifier") == self.identifier:
            intent = self.event.context.get("intent")
            effects = self.on_payment_processor_selected(intent=intent)
            return [effect.apply() for effect in effects]

        return []

    def _charge(self) -> Effect | None:
        """Handle the event when a charge is made."""
        if self.event.context.get("identifier") == self.identifier:
            patient = (
                Patient.objects.get(id=self.event.context.get("patient", {}).get("id"))
                if self.event.context.get("patient")
                else None
            )

            amount = Decimal(str(self.event.context.get("amount")))
            token = str(self.event.context.get("token"))
            additional_context = self.event.context.get("additional_context")

            try:
                additional_context = json.loads(additional_context)  # type: ignore[arg-type]
                if not isinstance(additional_context, dict):
                    additional_context = {"additional_context": additional_context}
            except (json.JSONDecodeError, TypeError):
                additional_context = {"additional_context": additional_context}

            effect = self.charge(amount=amount, token=token, patient=patient, **additional_context)

            return effect.apply() if effect else None

        return None

    def _payment_methods(self) -> list[Effect]:
        """List payment methods for the card payment processor."""
        if self.event.context.get("identifier") == self.identifier:
            patient = (
                Patient.objects.get(id=self.event.context.get("patient", {}).get("id"))
                if self.event.context.get("patient")
                else None
            )
            effects = self.payment_methods(patient=patient)
            return [effect.apply() for effect in effects]

        return []

    def _add_payment_method(self) -> Effect | None:
        """Handle the event when a card is added."""
        if self.event.context.get("identifier") == self.identifier:
            patient = (
                Patient.objects.get(id=self.event.context.get("patient", {}).get("id"))
                if self.event.context.get("patient")
                else None
            )

            if not patient:
                return None

            token = str(self.event.context.get("token"))
            additional_context = self.event.context.get("additional_context")

            try:
                additional_context = json.loads(additional_context)  # type: ignore[arg-type]
                if not isinstance(additional_context, dict):
                    additional_context = {"additional_context": additional_context}
            except (json.JSONDecodeError, TypeError):
                additional_context = {"additional_context": additional_context}

            effect = self.add_payment_method(token=token, patient=patient, **additional_context)
            return effect.apply() if effect else None

        return None

    def _remove_payment_method(self) -> Effect | None:
        """Handle the event when a payment method is removed."""
        if self.event.context.get("identifier") == self.identifier:
            patient = (
                Patient.objects.get(id=self.event.context.get("patient", {}).get("id"))
                if self.event.context.get("patient")
                else None
            )

            if not patient:
                return None

            token = str(self.event.context.get("token"))
            effect = self.remove_payment_method(token=token, patient=patient)
            return effect.apply() if effect else None

        return None

    def on_payment_processor_selected(self, intent: str | None) -> list[PaymentProcessorForm]:
        """Handle the event when a payment processor is selected."""
        patient = (
            Patient.objects.get(id=self.event.context.get("patient", {}).get("id"))
            if self.event.context.get("patient")
            else None
        )
        match intent:
            case self.PaymentIntent.ADD_CARD:
                return [self.add_card_form(patient)]
            case self.PaymentIntent.PAY:
                return [self.payment_form(patient)]
            case None:
                return [self.payment_form(patient), self.add_card_form(patient)]
            case _:
                return []

    @abstractmethod
    def payment_form(self, patient: Patient | None = None) -> PaymentProcessorForm:
        """Return the payment form for the credit card processor.

        Args:
            patient (Patient | None): The patient for whom the payment is being processed.

        Returns:
            PaymentProcessorForm: The form for processing the payment.
        """
        raise NotImplementedError("Subclasses must implement the payment_form method.")

    @abstractmethod
    def add_card_form(self, patient: Patient | None = None) -> PaymentProcessorForm:
        """Return the form for adding a card.

        Args:
            patient (Patient | None): The patient for whom the add is being added.

        Returns:
            PaymentProcessorForm: The form for adding a card.
        """
        raise NotImplementedError("Subclasses must implement the add_card_form method.")

    @abstractmethod
    def charge(
        self, amount: Decimal, token: str, patient: Patient | None = None, **kwargs: Any
    ) -> CardTransaction:
        """Charge a credit/debit card using the provided token.

        Args:
            amount (Decimal): The amount to charge.
            token (str): The token representing the credit card.
            patient (Patient | None): The patient for whom the charge is being made.
            **kwargs (Any): Additional context for the charge.

        Returns:
            CardTransaction: The result of the card transaction.
        """
        raise NotImplementedError("Subclasses must implement the charge method.")

    @abstractmethod
    def payment_methods(self, patient: Patient | None = None) -> list[PaymentMethod]:
        """List payment methods for the card payment processor.

        Args:
            patient (Patient | None): The patient for whom the payment methods are being listed.

        Returns:
            list[PaymentMethod]: A list of payment methods available for the card payment processor.
        """
        raise NotImplementedError("Subclasses must implement the payment_methods method.")

    @abstractmethod
    def add_payment_method(
        self, token: str, patient: Patient, **kwargs: Any
    ) -> AddPaymentMethodResponse:
        """Add a payment method for the card payment processor.

        Args:
            token (str): The token representing the payment method.
            patient (Patient): The patient for whom the payment method is being added.
            **kwargs (Any): Additional context for adding the payment method.

        Returns:
            AddPaymentMethodResponse: The response indicating the result of the addition operation.
        """
        raise NotImplementedError("Subclasses must implement the add_payment_method method.")

    @abstractmethod
    def remove_payment_method(self, token: str, patient: Patient) -> RemovePaymentMethodResponse:
        """Remove a payment method for the card payment processor.

        Args:
            token (str): The token representing the payment method to be removed.
            patient (Patient): The patient for whom the payment method is being removed.

        Returns:
            RemovePaymentMethodResponse: The response indicating the result of the removal operation.
        """
        raise NotImplementedError("Subclasses must implement the remove_payment_method method.")


__exports__ = ("CardPaymentProcessor",)



from datetime import date
from typing import Literal
from uuid import UUID

from canvas_sdk.base import Model
from canvas_sdk.effects import Effect
from canvas_sdk.effects.claim.claim_banner_alert import (
    BannerAlertIntent,
    _AddClaimBannerAlert,
    _RemoveClaimBannerAlert,
)
from canvas_sdk.effects.claim.claim_comment import _AddClaimComment
from canvas_sdk.effects.claim.claim_label import ColorEnum, Label, _AddClaimLabel, _RemoveClaimLabel
from canvas_sdk.effects.claim.claim_metadata import _ClaimMetadata
from canvas_sdk.effects.claim.claim_provider import (
    ClaimBillingProvider,
    ClaimFacility,
    ClaimOrderingProvider,
    ClaimProvider,
    ClaimReferringProvider,
    _UpdateClaimProvider,
)
from canvas_sdk.effects.claim.claim_queue import _MoveClaimToQueue
from canvas_sdk.effects.claim.payment.base import (
    ClaimAllocation,
    LineItemTransaction,
    PaymentMethod,
)
from canvas_sdk.effects.claim.payment.claim_payment import _PostClaimPayment


class ClaimEffect(Model):
    """
    Effect for performing actions on a Claim.

    Attributes:
        claim_id (UUID | str): The unique identifier for the claim instance.
    """

    claim_id: UUID | str

    def update_provider(
        self,
        billing_provider: ClaimBillingProvider | None = None,
        provider: ClaimProvider | None = None,
        referring_provider: ClaimReferringProvider | None = None,
        ordering_provider: ClaimOrderingProvider | None = None,
        facility: ClaimFacility | None = None,
    ) -> Effect:
        """
        Updates provider information for the claim.

        Args:
            billing_provider (ClaimBillingProvider | None): Billing provider information.
            provider (ClaimProvider | None): Rendering or attending provider information.
            referring_provider (ClaimReferringProvider | None): Referring provider information.
            ordering_provider (ClaimOrderingProvider | None): Ordering provider information.
            facility (ClaimFacility | None): Facility information.

        Returns:
            Effect: An effect that updates provider information for the claim.
        """
        return _UpdateClaimProvider(
            claim_id=self.claim_id,
            billing_provider=billing_provider,
            provider=provider,
            referring_provider=referring_provider,
            ordering_provider=ordering_provider,
            facility=facility,
        ).apply()

    def upsert_metadata(self, key: str, value: str) -> Effect:
        """
        Upserts a metadata record to the claim.

        Args:
            key (str): The key of the metadata.
            value (str): The value of the metadata.

        Returns:
            Effect: An effect that upserts the metadata record to the claim.
        """
        return _ClaimMetadata(claim_id=self.claim_id, key=key).upsert(value=value)

    def add_banner(
        self, key: str, narrative: str, intent: BannerAlertIntent, href: str | None = None
    ) -> Effect:
        """
        Adds a banner alert to a claim.

        Args:
            key (str): A unique identifier for the banner alert.
            narrative (str): The text content to display in the banner.
            intent (BannerAlertIntent): The intent/severity level of the banner (info, warning, or alert).
            href (str | None): Optional URL link for the banner. Defaults to None.

        Returns:
            Effect: An effect that adds the banner alert to the claim.
        """
        return _AddClaimBannerAlert(
            claim_id=self.claim_id, key=key, narrative=narrative, intent=intent, href=href
        ).apply()

    def remove_banner(self, key: str) -> Effect:
        """
        Removes a banner alert from a claim.

        Args:
            key (str): A unique identifier for the banner alert.

        Returns:
            Effect: An effect that removes the banner alert from the claim.
        """
        return _RemoveClaimBannerAlert(claim_id=self.claim_id, key=key).apply()

    def add_comment(self, comment: str) -> Effect:
        """
        Adds a comment to the claim.

        Args:
            comment (str): The comment text to add to the claim.

        Returns:
            Effect: An effect that adds the comment to the claim.
        """
        return _AddClaimComment(claim_id=self.claim_id, comment=comment).apply()

    def add_labels(self, labels: list[str | Label]) -> Effect:
        """
        Adds one or more labels to the claim.

        Args:
            labels (list[str | Label]): A list of label names (str) or Label objects with color and name.

        Returns:
            Effect: An effect that adds the labels to the claim.
        """
        return _AddClaimLabel(claim_id=self.claim_id, labels=labels).apply()

    def remove_labels(self, labels: list[str]) -> Effect:
        """
        Removes one or more labels from the claim.

        Args:
            labels (list[str]): A list of label names to remove.

        Returns:
            Effect: An effect that removes the labels from the claim.
        """
        return _RemoveClaimLabel(claim_id=self.claim_id, labels=labels).apply()

    def move_to_queue(self, queue: str) -> Effect:
        """
        Moves the claim to a queue.

        Args:
            queue (str): The name of the queue to move the claim to.

        Returns:
            Effect: An effect that moves the claim to the specified queue.
        """
        return _MoveClaimToQueue(claim_id=self.claim_id, queue=queue).apply()

    def post_payment(
        self,
        claim_coverage_id: str | UUID | Literal["patient"],
        line_item_transactions: list[LineItemTransaction],
        method: PaymentMethod,
        move_to_queue_name: str | None = None,
        claim_description: str | None = None,
        check_date: date | None = None,
        check_number: str | None = None,
        deposit_date: date | None = None,
        payment_description: str | None = None,
    ) -> Effect:
        """
        Posts a coverage or patient payment to the claim.

        Args:
            claim_coverage_id (str | UUID | Literal["patient"]): The coverage ID or "patient" for patient payments.
            line_item_transactions (list[LineItemTransaction]): The line item transactions for the payment.
            method (PaymentMethod): The payment method (CASH, CHECK, CARD, OTHER).
            move_to_queue_name (str | None): Optional queue name to move the claim to after posting.
            claim_description (str | None): Optional description for the claim allocation.
            check_date (date | None): Required when method is CHECK.
            check_number (str | None): Required when method is CHECK.
            deposit_date (date | None): Optional deposit date.
            payment_description (str | None): Optional payment description.

        Returns:
            Effect: An effect that posts the payment to the claim.
        """
        claim_allocation = ClaimAllocation(
            claim_id=self.claim_id,
            claim_coverage_id=claim_coverage_id,
            line_item_transactions=line_item_transactions,
            move_to_queue_name=move_to_queue_name,
            description=claim_description,
        )
        return _PostClaimPayment(
            claim=claim_allocation,
            method=method,
            check_date=check_date,
            check_number=check_number,
            deposit_date=deposit_date,
            payment_description=payment_description,
        ).apply()


__all__ = __exports__ = (
    "BannerAlertIntent",
    "ClaimEffect",
    "Label",
    "ColorEnum",
    "LineItemTransaction",
    "PaymentMethod",
    "ClaimBillingProvider",
    "ClaimFacility",
    "ClaimOrderingProvider",
    "ClaimProvider",
    "ClaimReferringProvider",
)



from canvas_sdk.clients.extend_ai.structures.classification import Classification
from canvas_sdk.clients.extend_ai.structures.processor_meta import ProcessorMeta
from canvas_sdk.clients.extend_ai.structures.processor_run import ProcessorRun
from canvas_sdk.clients.extend_ai.structures.processor_version import ProcessorVersion
from canvas_sdk.clients.extend_ai.structures.request_failed import RequestFailed
from canvas_sdk.clients.extend_ai.structures.stored_file import StoredFile

__all__ = __exports__ = (
    "Classification",
    "ProcessorMeta",
    "ProcessorRun",
    "ProcessorVersion",
    "RequestFailed",
    "StoredFile",
)



from canvas_sdk.clients.extend_ai.constants.base_processor import BaseProcessor
from canvas_sdk.clients.extend_ai.constants.parser_chunking import ParserChunking
from canvas_sdk.clients.extend_ai.constants.parser_target import ParserTarget
from canvas_sdk.clients.extend_ai.constants.processor_type import ProcessorType
from canvas_sdk.clients.extend_ai.constants.run_status import RunStatus
from canvas_sdk.clients.extend_ai.constants.split_method import SplitMethod
from canvas_sdk.clients.extend_ai.constants.version_name import VersionName

__all__ = __exports__ = (
    "BaseProcessor",
    "ParserChunking",
    "ParserTarget",
    "ProcessorType",
    "RunStatus",
    "SplitMethod",
    "VersionName",
)



from dataclasses import dataclass
from json import loads

from canvas_sdk.clients.sendgrid.structures.parsed_attachment import ParsedAttachment
from canvas_sdk.clients.sendgrid.structures.parsed_envelope import ParsedEnvelope
from canvas_sdk.clients.sendgrid.structures.parsed_header import ParsedHeader


@dataclass(frozen=True)
class ParsedEmail:
    """Represents a complete inbound email parsed by SendGrid's Inbound Parse Webhook."""

    headers: list[ParsedHeader]
    charsets: dict[str, str]
    envelope: ParsedEnvelope
    email_from: str
    email_to: str
    subject: str
    text: str
    html: str
    attachments: int
    attachment_info: dict[str, ParsedAttachment]
    content_ids: dict[str, str]
    spf: str
    dkim: str
    spam_report: list[str]
    spam_score: float

    @classmethod
    def from_dict(cls, data: dict) -> "ParsedEmail":
        """Create ParsedEmail instance from dictionary."""
        headers = [line.split(":") for line in data["headers"].split("\n")]

        attachments = 0
        if "attachments" in data:
            attachments = int(data["attachments"])
        attachment_info = {}
        if "attachment_info" in data:
            attachment_info = {
                key: ParsedAttachment.from_dict(info)
                for key, info in loads(data["attachment_info"]).items()
            }
        content_ids = {}
        if "content-ids" in data:
            content_ids = loads(data["content-ids"])
        spam_score = 0.0
        if "spam_score" in data:
            spam_score = float(data["spam_score"])
        spam_report = []
        if "spam_report" in data:
            spam_report = data["spam_report"].split("\n")

        return cls(
            headers=[
                ParsedHeader(name=header[0].strip(), value=":".join(header[1:]).strip())
                for header in headers
            ],
            charsets=loads(data["charsets"]),
            envelope=ParsedEnvelope.from_dict(loads(data["envelope"])),
            email_from=data["from"],
            email_to=data["to"],
            subject=data["subject"],
            text=data["text"],
            html=data["html"],
            attachments=attachments,
            attachment_info=attachment_info,
            content_ids=content_ids,
            spf=data["SPF"],
            dkim=data["dkim"],
            spam_report=spam_report,
            spam_score=spam_score,
        )

    def to_dict(self) -> dict:
        """Convert parsed email to dictionary format."""
        return {
            "headers": [h.to_dict() for h in self.headers],
            "charsets": self.charsets,
            "envelope": self.envelope.to_dict(),
            "emailFrom": self.email_from,
            "emailTo": self.email_to,
            "subject": self.subject,
            "text": self.text,
            "html": self.html,
            "attachments": self.attachments,
            "attachmentInfo": {key: info.to_dict() for key, info in self.attachment_info.items()},
            "contentIds": self.content_ids,
            "spf": self.spf,
            "dkim": self.dkim,
            "spamReport": self.spam_report,
            "spamScore": str(round(self.spam_score, 2)),
        }


__exports__ = ()



from dataclasses import dataclass
from typing import Self

from canvas_sdk.clients.extend_ai.structures.structure import Structure


@dataclass(frozen=True)
class Classification(Structure):
    """Represents a document classification result.

    Attributes:
        id: The unique identifier for this classification.
        type: The classification type or category.
        description: A description of the classification.
    """

    id: str
    type: str
    description: str

    @classmethod
    def from_dict(cls, data: dict) -> Self:
        """Create a Classification instance from a dictionary.

        Args:
            data: Dictionary containing classification data from the API.

        Returns:
            A new Classification instance.
        """
        return cls(
            id=data["id"],
            type=data["type"],
            description=data["description"],
        )

    def to_dict(self) -> dict:
        """Convert this Classification to a dictionary.

        Returns:
            Dictionary representation suitable for API requests.
        """
        return {
            "id": self.id,
            "type": self.type,
            "description": self.description,
        }


__exports__ = ()



from enum import Enum


class MessageStatus(Enum):
    """Enum representing the various statuses a Twilio message can have throughout its lifecycle."""

    ACCEPTED = "accepted"
    SCHEDULED = "scheduled"
    CANCELED = "canceled"
    QUEUED = "queued"
    SENDING = "sending"
    SENT = "sent"
    FAILED = "failed"
    DELIVERED = "delivered"
    UNDELIVERED = "undelivered"
    PARTIALLY_DELIVERED = "partially_delivered"
    RECEIVING = "receiving"
    RECEIVED = "received"
    READ = "read"


__exports__ = ()



import json

from canvas_sdk.commands import GoalCommand
from canvas_sdk.effects import Effect
from canvas_sdk.events import EventType
from canvas_sdk.handlers import BaseHandler
from canvas_sdk.v1.data.note import Note
from canvas_sdk.v1.data.patient import Patient
from logger import log

# Inherit from BaseHandler to properly get registered for events
class NewOfficeVisitNoteHandler(BaseHandler):
    """Originates goal command when a new office visit note is created."""

    # Name the event type you wish to run in response to
    RESPONDS_TO = EventType.Name(EventType.NOTE_STATE_CHANGE_EVENT_CREATED)

    def compute(self) -> list[Effect]:
        """This method gets called when an event of the type RESPONDS_TO is fired."""
        # This class is initialized with several pieces of information you can
        # access.
        #
        # `self.event` is the event object that caused this method to be
        # called.
        #
        # `self.event.target.id` is an identifier for the object that is the subject of
        # the event. In this case, it would be the identifier of the note state.
        #
        # `self.event.context` is a python dictionary of additional data that was
        # given with the event. The information given here depends on the
        # event type.
        #
        # `self.secrets` is a python dictionary of the secrets you defined in
        # your CANVAS_MANIFEST.json and set values for in the uploaded
        # plugin's configuration page: <emr_base_url>/admin/plugin_io/plugin/<plugin_id>/change/
        # Example: self.secrets['WEBHOOK_URL']

        # You can log things and see them using the Canvas CLI's log streaming
        # function.
        log.info(f"[NewOfficeVisitNoteHandler] Context: {self.event.context}")

        # Get the note state from context
        note_state = self.event.context.get("state")

        # Check if the note state is NEW
        if note_state != "NEW":
            return []

        # Get the note ID from context and fetch the Note object
        note_id = self.event.context.get("note_id")
        note = Note.objects.get(id=note_id)

        # Check if note type is OFFICE VISIT
        note_type_name = note.note_type_version.name

        if note_type_name != "Office visit":
            return []

        # Get the note UUID from context (it's already a UUID string)
        note_uuid = note_id

        # Get the patient to create a personalized goal statement
        patient_id = self.event.context.get("patient_id")
        patient = Patient.objects.get(id=patient_id)
        patient_name = patient.first_name
        goal_statement = f"{patient_name} will build plugins with the Canvas SDK to improve their clinical workflow"

        # Create and originate Goal command with personalized statement
        goal_command = GoalCommand(note_uuid=note_uuid, goal_statement=goal_statement)

        return [goal_command.originate()]



from dataclasses import dataclass
from datetime import datetime
from typing import Self

from canvas_sdk.clients.twilio.constants.constants import Constants
from canvas_sdk.clients.twilio.constants.message_direction import MessageDirection
from canvas_sdk.clients.twilio.constants.message_status import MessageStatus
from canvas_sdk.clients.twilio.structures.helper import Helper
from canvas_sdk.clients.twilio.structures.structure import Structure


@dataclass(frozen=True)
class Message(Structure):
    """Represents a Twilio message with all its associated metadata and status information."""

    sid: str
    body: str
    date_created: datetime
    date_sent: datetime | None
    date_updated: datetime
    direction: MessageDirection
    number_from: str
    number_to: str
    price: str | None  # float
    price_unit: str
    error_code: int | None
    error_message: str | None
    uri: str
    count_media: int | None
    count_segments: int
    status: MessageStatus
    sub_resource_uris: dict[str, str] | None

    @classmethod
    def from_dict(cls, data: dict) -> Self:
        """Create a Message instance from a dictionary.

        Args:
            data: Dictionary containing the message data from Twilio API.

        Returns:
            Message instance populated with the provided data.
        """
        return cls(
            sid=data["sid"],
            body=data["body"],
            date_created=Helper.to_datetime(data, "date_created"),
            date_sent=Helper.to_datetime_or_none(data, "date_sent"),
            date_updated=Helper.to_datetime(data, "date_updated"),
            direction=MessageDirection(data["direction"]),
            number_from=data["from"],
            number_to=data["to"],
            price=data.get("price"),
            price_unit=data["price_unit"],
            error_code=data.get("error_code"),
            error_message=data.get("error_message"),
            uri=data["uri"],
            count_media=data["num_media"],
            count_segments=data["num_segments"],
            status=MessageStatus(data["status"]),
            sub_resource_uris=data["subresource_uris"],
        )

    def to_dict(self) -> dict:
        """Convert the Message to a dictionary representation.

        Returns:
            Dictionary containing all message data in both Twilio and standard formats.
        """
        return {
            "sid": self.sid,
            "body": self.body,
            "date_created": Helper.from_datetime(self.date_created, Constants.twilio_date),
            "date_sent": Helper.from_datetime_or_none(self.date_sent, Constants.twilio_date),
            "date_updated": Helper.from_datetime(self.date_updated, Constants.twilio_date),
            "created": Helper.from_datetime(self.date_created, Constants.standard_date),
            "sent": Helper.from_datetime_or_none(self.date_sent, Constants.standard_date),
            "updated": Helper.from_datetime(self.date_updated, Constants.standard_date),
            "direction": self.direction.value,
            "from": self.number_from,
            "to": self.number_to,
            "price": self.price,
            "price_unit": self.price_unit,
            "error_code": self.error_code,
            "error_message": self.error_message,
            "uri": self.uri,
            "num_media": self.count_media,
            "num_segments": self.count_segments,
            "status": self.status.value,
            "subresource_uris": self.sub_resource_uris,
        }


__exports__ = ()



from typing import get_args, get_origin

from pydantic import BaseModel, ConfigDict
from pydantic.alias_generators import to_camel


class BaseModelLlmJson(BaseModel):
    """Subclass of BaseModel to ensure valid JSON Schema.

    When using the method `model_json_schema`:
    - the key `additionalProperties` will be present, set to False
    - the fields name will be camelCased
    """

    model_config = ConfigDict(extra="forbid", alias_generator=to_camel)

    @classmethod
    def validate_nested_models(cls) -> bool:
        """Validate that all nested BaseModel fields are BaseModelLlmJson subclasses."""
        for _, field_info in cls.model_fields.items():
            field_type = field_info.annotation

            types_to_check: list = []
            if get_origin(field_type) is not None:
                types_to_check.extend(get_args(field_type))
            else:
                types_to_check.append(field_type)

            for a_type in types_to_check:
                if not issubclass(a_type, BaseModel):
                    continue
                if not (issubclass(a_type, BaseModelLlmJson) and a_type.validate_nested_models()):
                    return False
        return True


__exports__ = ()



from __future__ import annotations

import json
from http import HTTPStatus

from requests import exceptions

from canvas_sdk.clients.llms.constants.file_type import FileType
from canvas_sdk.clients.llms.libraries.llm_api import LlmApi
from canvas_sdk.clients.llms.structures.llm_response import LlmResponse
from canvas_sdk.clients.llms.structures.llm_tokens import LlmTokens


class LlmOpenai(LlmApi):
    """OpenAI LLM API client.

    Implements the LlmBase interface for OpenAI's API.
    """

    def to_dict(self) -> dict:
        """Convert prompts and add the necessary information to OpenAI API request format.

        Returns:
            Dictionary formatted for OpenAI API with instructions and input messages.
        """
        roles = {
            self.ROLE_SYSTEM: "developer",
            self.ROLE_USER: "user",
            self.ROLE_MODEL: "assistant",
        }
        messages: list[dict] = [
            {
                "role": roles[prompt.role],
                "content": [
                    {
                        "type": "input_text" if prompt.role == self.ROLE_USER else "output_text",
                        "text": "\n".join(prompt.text),
                    }
                ],
            }
            for prompt in self.prompts
            if prompt.role != self.ROLE_SYSTEM
        ]
        # if there are files and the last message has the user's role
        if messages and messages[-1]["role"] == roles[self.ROLE_USER]:
            for file_url in self.file_urls:
                item = {}
                if file_url.type == FileType.PDF:
                    item = {"type": "input_file", "file_url": file_url.url}
                elif file_url.type == FileType.IMAGE:
                    item = {"type": "input_image", "image_url": file_url.url}
                if item:
                    messages[-1]["content"].append(item)

            for file_content in self.file_contents:
                if file_content.mime_type.startswith("image/"):
                    messages[-1]["content"].append(
                        {
                            "type": "input_image",
                            "image_url": f"data:{file_content.mime_type};base64,{file_content.content.decode('utf-8')}",
                        }
                    )
                elif file_content.mime_type.endswith("/pdf"):
                    messages[-1]["content"].append(
                        {
                            "type": "input_file",
                            "filename": "inputFile",
                            "file_data": f"data:{file_content.mime_type};base64,{file_content.content.decode('utf-8')}",
                        }
                    )

            self.file_urls = []
            self.file_contents = []

        system_prompt = "\n".join(
            ["\n".join(prompt.text) for prompt in self.prompts if prompt.role == self.ROLE_SYSTEM]
        )
        # structured output requested
        structured = {}
        if self.schema:
            structured = {
                "text": {
                    "format": {
                        "type": "json_schema",
                        "name": self.schema.__name__,
                        "strict": True,
                        "schema": self.schema.model_json_schema(),
                    }
                }
            }
        return (
            self.settings.to_dict()
            | structured
            | {"instructions": system_prompt, "input": messages}
        )

    @classmethod
    def _api_base_url(cls) -> str:
        return "https://us.api.openai.com"

    def request(self) -> LlmResponse:
        """Make a request to the OpenAI API.

        Returns:
            Response containing status code, generated text, and token usage.
        """
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.settings.api_key}",
        }
        data = json.dumps(self.to_dict())

        tokens = LlmTokens(prompt=0, generated=0)
        try:
            request = self.http.post("/v1/responses", headers=headers, data=data)
            code = request.status_code
            response = request.text
            if code == HTTPStatus.OK.value:
                content = json.loads(request.text)
                response = ""
                for output in content.get("output", [{}]):
                    if output.get("type", "") == "message":
                        response += output.get("content", [{}])[0].get("text", "")
                usage = content.get("usage", {})
                tokens = LlmTokens(
                    prompt=usage.get("input_tokens") or 0,
                    generated=usage.get("output_tokens") or 0,
                )
        except exceptions.RequestException as e:
            code = HTTPStatus.BAD_REQUEST
            response = f"Request failed: {e}"
            if message := getattr(e, "response", None):
                code = message.status_code
                response = message.text

        return LlmResponse(
            code=HTTPStatus(code),
            response=response,
            tokens=tokens,
        )


__exports__ = ("LlmOpenai",)



import datetime

import factory
from django.utils import timezone
from factory.fuzzy import FuzzyDate

from canvas_sdk.v1.data import UncategorizedClinicalDocument, UncategorizedClinicalDocumentReview


class UncategorizedClinicalDocumentReviewFactory(
    factory.django.DjangoModelFactory[UncategorizedClinicalDocumentReview]
):
    """Factory for creating UncategorizedClinicalDocumentReview."""

    class Meta:
        model = UncategorizedClinicalDocumentReview

    internal_comment = factory.Faker("paragraph")
    message_to_patient = factory.Faker("sentence", nb_words=20)
    status = factory.Faker("random_element", elements=["pending", "reviewed", "completed"])
    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")
    patient_communication_method = factory.Faker(
        "random_element", elements=["email", "phone", "portal", "mail"]
    )


class UncategorizedClinicalDocumentFactory(
    factory.django.DjangoModelFactory[UncategorizedClinicalDocument]
):
    """Factory for creating UncategorizedClinicalDocument."""

    class Meta:
        model = UncategorizedClinicalDocument

    originator = factory.SubFactory("canvas_sdk.test_utils.factories.CanvasUserFactory")
    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")
    review = factory.SubFactory(UncategorizedClinicalDocumentReviewFactory)
    assigned_by = factory.SubFactory("canvas_sdk.test_utils.factories.CanvasUserFactory")

    name = factory.Faker("sentence", nb_words=3)
    review_mode = factory.Faker("random_element", elements=["IN", "OT"])
    requires_signature = factory.Faker("boolean")
    assigned_date = factory.LazyFunction(lambda: timezone.now())
    original_date = FuzzyDate(
        start_date=datetime.date.today() - datetime.timedelta(days=365),
        end_date=datetime.date.today(),
    )
    comment = factory.Faker("paragraph")



from enum import Enum


class FileType(Enum):
    """Enumeration of supported file types for LLM file attachments.

    Attributes:
        IMAGE: Image file type (e.g., PNG, JPEG, GIF).
        PDF: PDF document file type.
        TEXT: Plain text file type.
    """

    IMAGE = "image"
    PDF = "pdf"
    TEXT = "text"


__exports__ = ()



import factory

from canvas_sdk.v1.data import Prescription


class PrescriptionFactory(factory.django.DjangoModelFactory[Prescription]):
    """Factory for Prescription."""

    class Meta:
        model = Prescription

    note = factory.SubFactory(
        "canvas_sdk.test_utils.factories.NoteFactory",
        patient=factory.SelfAttribute("..patient"),
        originator=factory.SelfAttribute("..originator"),
    )
    originator = factory.SubFactory("canvas_sdk.test_utils.factories.CanvasUserFactory")
    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")
    prescriber = factory.SubFactory("canvas_sdk.test_utils.factories.StaffFactory")
    supervising_provider = factory.SubFactory("canvas_sdk.test_utils.factories.StaffFactory")

    count_of_refills_allowed = 1
    dispense_quantity = 4
    dose_form = "tablet"
    dose_frequency = 2.2
    dose_frequency_interval = "day"
    dose_quantity = 1.1
    dose_route = "by mouth"
    duration_in_days = 22
    generic_substitutions_allowed = False
    message_id = "2944ae21dbab4cdb9f3a9fee6c184efa"
    pharmacy_address = "1234 Main Street"
    pharmacy_is_read_only = False
    pharmacy_name = "Walgreens"
    pharmacy_ncpdp_id = "1367084"
    pharmacy_phone_number = "4155353439"
    sig_original_input = "1 tab bid po"



from dataclasses import dataclass
from datetime import UTC, datetime

from canvas_sdk.clients.sendgrid.structures.address import Address
from canvas_sdk.clients.sendgrid.structures.attachment import Attachment
from canvas_sdk.clients.sendgrid.structures.body_content import BodyContent
from canvas_sdk.clients.sendgrid.structures.recipient import Recipient


@dataclass(frozen=True)
class Email:
    """Represents a complete email message with all components."""

    sender: Address
    reply_tos: list[Address]
    recipients: list[Recipient]
    subject: str
    bodies: list[BodyContent]
    attachments: list[Attachment]
    send_at: int

    @classmethod
    def now(cls) -> int:
        """Get current timestamp for immediate sending."""
        return cls.timestamp(datetime.now(tz=UTC))

    @classmethod
    def timestamp(cls, now: datetime) -> int:
        """Convert datetime to Unix timestamp."""
        return int(now.timestamp())


__exports__ = ()



from abc import abstractmethod
from dataclasses import dataclass

from canvas_sdk.clients.extend_ai.constants.processor_type import ProcessorType
from canvas_sdk.clients.extend_ai.structures.structure import Structure


@dataclass(frozen=True)
class ConfigBase(Structure):
    """Abstract base class for processor configurations.

    All processor configurations must inherit from this class and implement
    the processor_type method to specify their processor type.
    """

    @classmethod
    @abstractmethod
    def processor_type(cls) -> ProcessorType:
        """Get the processor type for this configuration.

        Returns:
            The ProcessorType enum value for this configuration.
        """
        ...


__exports__ = ()



import factory

from canvas_sdk.v1.data import Language, Letter, LetterActionEvent
from canvas_sdk.v1.data.letter import EventTypeChoices


class LanguageFactory(factory.django.DjangoModelFactory[Language]):
    """Factory for creating Language."""

    class Meta:
        model = Language

    code = factory.Sequence(lambda n: f"l{n:02d}")
    description = factory.Faker("language_name")


class LetterFactory(factory.django.DjangoModelFactory[Letter]):
    """Factory for creating Letter."""

    class Meta:
        model = Letter

    content = factory.Faker("paragraph")
    printed = None
    note = factory.SubFactory("canvas_sdk.test_utils.factories.NoteFactory")
    staff = factory.SubFactory("canvas_sdk.test_utils.factories.StaffFactory")


class LetterActionEventFactory(factory.django.DjangoModelFactory[LetterActionEvent]):
    """Factory for creating LetterActionEvent."""

    class Meta:
        model = LetterActionEvent

    event_type = EventTypeChoices.PRINTED
    send_fax_id = factory.Faker("uuid4")
    received_by_fax = factory.Faker("boolean")
    delivered_by_fax = factory.Faker("boolean")
    fax_result_msg = factory.Faker("paragraph")
    originator = factory.SubFactory("canvas_sdk.test_utils.factories.CanvasUserFactory")
    letter = factory.SubFactory("canvas_sdk.test_utils.factories.LetterFactory")



from decimal import Decimal
from typing import Self

from django.db import models

from canvas_sdk.v1.data.base import IdentifiableModel, TimestampedModel
from canvas_sdk.v1.data.utils import quantize


class PostingQuerySet(models.QuerySet):
    """A queryset for CoveragePosting objects."""

    def active(self) -> Self:
        """Return a queryset that filters for active postings."""
        return self.filter(entered_in_error__isnull=True)


class BasePosting(TimestampedModel):
    """
    Base model to aggregate multiple line item transactions on a claim.
    """

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_baseposting_001"

    objects = PostingQuerySet.as_manager()

    corrected_posting = models.ForeignKey(
        "v1.BasePosting", related_name="correction_postings", on_delete=models.PROTECT, null=True
    )

    claim = models.ForeignKey("v1.Claim", related_name="postings", on_delete=models.PROTECT)
    payment_collection = models.ForeignKey(
        "v1.PaymentCollection", related_name="postings", null=True, on_delete=models.PROTECT
    )

    description = models.TextField(default="")

    entered_in_error = models.ForeignKey(
        "v1.CanvasUser", on_delete=models.PROTECT, null=True, blank=True
    )

    @property
    def paid_amount(self) -> Decimal:
        """Returns the total amount paid for this posting."""
        return quantize(sum(lip.amount for lip in self.newlineitempayments.all()))

    @property
    def contractual_adjusted_amount(self) -> Decimal:
        """Returns the total amount of contractual adjustments for this posting."""
        return quantize(
            sum(lia.amount for lia in self.newlineitemadjustments.all() if lia.write_off)
        )

    @property
    def non_write_off_adjusted_amount(self) -> Decimal | int:
        """Returns the total amount of non-write-off adjustments for this posting."""
        return sum(lia.amount for lia in self.newlineitemadjustments.all() if not lia.write_off)

    @property
    def transferred_amount(self) -> Decimal:
        """Returns the total amount transferred for this posting."""
        return quantize(sum(lit.amount for lit in self.lineitemtransfers.all()))

    @property
    def transferred_to_patient_amount(self) -> Decimal | int:
        """Returns the total amount transferred to the patient."""
        transfers = [trans for trans in self.lineitemtransfers.all() if trans.transfer_to_patient]
        return sum(lit.amount for lit in transfers)

    @property
    def transferred_to_coverage_amount(self) -> Decimal | int:
        """Returns the total amount transferred to another coverage."""
        transfers = [trans for trans in self.lineitemtransfers.all() if trans.transfer_to]
        return sum(lit.amount for lit in transfers)

    @property
    def adjusted_and_transferred_amount(self) -> Decimal:
        """Returns the charges amount per claim for a patient posting."""
        return self.contractual_adjusted_amount + self.transferred_amount

    @property
    def posted_amount(self) -> Decimal:
        """Property to compute how much was posted with this posting: sums all payments and write-off adjustments."""
        return self.paid_amount + self.contractual_adjusted_amount + self.transferred_amount


class CoveragePosting(BasePosting):
    """Represents an insurance payment/adjustment on a claim level."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_coverageposting_001"

    remittance = models.ForeignKey(
        "v1.BaseRemittanceAdvice", related_name="postings", on_delete=models.PROTECT, null=True
    )
    claim_coverage = models.ForeignKey(
        "v1.ClaimCoverage", related_name="postings", on_delete=models.PROTECT
    )

    crossover_carrier = models.CharField(max_length=255)
    crossover_id = models.CharField(max_length=255)
    payer_icn = models.CharField(max_length=255)
    position_in_era = models.PositiveIntegerField()


class PatientPosting(BasePosting):
    """A model that represents a patient payment/adjustment on a claim level."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_patientposting_001"

    claim_patient = models.ForeignKey(
        "v1.ClaimPatient", related_name="postings", on_delete=models.PROTECT
    )
    patient_payment = models.ForeignKey(
        "v1.BulkPatientPosting", related_name="postings", on_delete=models.PROTECT, null=True
    )

    copay = models.ForeignKey(
        "v1.BulkPatientPosting", related_name="copays", on_delete=models.PROTECT, null=True
    )

    @property
    def discounted_amount(self) -> Decimal:
        """Returns the discounted amount per claim for a patient bulk posting."""
        if not self.patient_payment or not self.patient_payment.discount:
            return Decimal("0")

        discount = self.patient_payment.discount
        return quantize(
            sum(
                lia.amount
                for lia in self.newlineitemadjustments.all()
                if (
                    lia.write_off
                    and (lia.group, lia.code)
                    == (discount.adjustment_group, discount.adjustment_code)
                )
            )
        )

    @property
    def charges_amount(self) -> Decimal:
        """Returns the charges amount per claim for a patient posting."""
        return self.discounted_amount + self.paid_amount


class AbstractBulkPosting(TimestampedModel, IdentifiableModel):
    """Patient and Insurance bulk posting shared columns."""

    class Meta:
        abstract = True

    payment_collection = models.OneToOneField(
        "v1.PaymentCollection", related_name="%(class)s", on_delete=models.PROTECT
    )

    total_paid = models.DecimalField(max_digits=8, decimal_places=2)


class BulkPatientPosting(AbstractBulkPosting):
    """Model to aggregate bulk patient payments on multiple claims."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_bulkpatientposting_001"

    discount = models.ForeignKey(
        "v1.Discount", related_name="patient_postings", null=True, on_delete=models.SET_NULL
    )
    payer = models.ForeignKey("v1.Patient", related_name="payments", on_delete=models.PROTECT)

    @property
    def total_posted_amount(self) -> Decimal:
        """Property to compute how much was posted with all associated postings."""
        postings = self.postings.active()
        return quantize(sum(posting.posted_amount for posting in postings))

    @property
    def discounted_amount(self) -> Decimal:
        """Returns the sum of discounted amounts for every posting created with this collection."""
        postings = self.postings.active()
        return quantize(sum(posting.discounted_amount for posting in postings))


class BaseRemittanceAdvice(AbstractBulkPosting):
    """Manual and Electronic Shared columns."""

    class Meta:
        db_table = "canvas_sdk_data_quality_and_revenue_baseremittanceadvice_001"

    transactor = models.ForeignKey(
        "v1.Transactor", related_name="remits", null=True, on_delete=models.PROTECT
    )

    era_id = models.CharField(max_length=255, null=True, blank=True, db_index=True)

    @property
    def total_posted_amount(self) -> Decimal:
        """Property to compute how much was posted with all associated postings."""
        postings = self.postings.active()
        return quantize(sum(posting.posted_amount for posting in postings))


__exports__ = (
    "BasePosting",
    "CoveragePosting",
    "PatientPosting",
    "BulkPatientPosting",
    "BaseRemittanceAdvice",
)



from dataclasses import dataclass
from typing import Any
from uuid import UUID

from pydantic import conlist
from pydantic_core import InitErrorDetails

from canvas_sdk.effects.base import EffectType, _BaseEffect
from canvas_sdk.v1.data import Claim
from canvas_sdk.v1.data.common import ColorEnum


@dataclass
class Label:
    """A class representing a label."""

    color: ColorEnum
    name: str

    def to_dict(self) -> dict[str, Any]:
        """Convert the label to a dictionary."""
        return {"color": self.color.value, "name": self.name}


class _ClaimLabelBase(_BaseEffect):
    """Base class for managing ClaimLabels."""

    claim_id: UUID | str
    labels: conlist(str | Label, min_length=1)  # type: ignore

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)
        if not Claim.objects.filter(id=self.claim_id).exists():
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Claim with id {self.claim_id} does not exist.",
                    self.claim_id,
                )
            )
        return errors


class _AddClaimLabel(_ClaimLabelBase):
    """Effect to add a label to a Claim."""

    class Meta:
        effect_type = EffectType.ADD_CLAIM_LABEL

    @property
    def values(self) -> dict[str, Any]:
        """The values for adding a claim label."""
        return {
            "claim_id": str(self.claim_id),
            "labels": [
                label.to_dict() if isinstance(label, Label) else {"name": label}
                for label in self.labels
            ],
        }


class _RemoveClaimLabel(_ClaimLabelBase):
    """Effect to remove a label from a Claim."""

    class Meta:
        effect_type = EffectType.REMOVE_CLAIM_LABEL

    labels: conlist(str, min_length=1)  # type: ignore

    @property
    def values(self) -> dict[str, Any]:
        """The values for removing a claim label."""
        return {"claim_id": str(self.claim_id), "labels": list(self.labels)}


__exports__ = ()



from ..value_set import ValueSet

class EstimatedGestationalAgeAtDelivery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to describe concepts of assessment for estimated gestational age at delivery.

    **Data Element Scope:** This value set may use a model element related to assessment.

    **Inclusion Criteria:** Includes concepts that define an assessment of the estimated gestational age at delivery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Estimated Gestational Age at Delivery"
    OID = "2.16.840.1.113762.1.4.1045.26"
    DEFINITION_VERSION = "20210611"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "11884-4",  # Gestational age Estimated
        "11885-1",  # Gestational age Estimated from last menstrual period
        "18185-9",  # Gestational age
        "49051-6",  # Gestational age in weeks
    }

class NonInvasiveOxygenTherapy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for oxygen therapies not associated with invasive mechanical ventilation

    **Data Element Scope:** This value set may use a model element related to Assessment

    **Inclusion Criteria:** Includes concepts that represent non invasive oxygen therapies including oxygen masks, nasal cannulas, CPAP, and BPAP which are not associated with invasive mechanical ventilation

    **Exclusion Criteria:** Oxygen therapies and devices associated with invasive mechanical ventilation
    """

    VALUE_SET_NAME = "Non Invasive Oxygen Therapy"
    OID = "2.16.840.1.113762.1.4.1248.213"
    DEFINITION_VERSION = "20230512"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "5A09357",  # Assistance with Respiratory Ventilation, Less than 24 Consecutive Hours, Continuous Positive Airway Pressure
        "5A09358",  # Assistance with Respiratory Ventilation, Less than 24 Consecutive Hours, Intermittent Positive Airway Pressure
        "5A09457",  # Assistance with Respiratory Ventilation, 24-96 Consecutive Hours, Continuous Positive Airway Pressure
        "5A09458",  # Assistance with Respiratory Ventilation, 24-96 Consecutive Hours, Intermittent Positive Airway Pressure
        "5A09557",  # Assistance with Respiratory Ventilation, Greater than 96 Consecutive Hours, Continuous Positive Airway Pressure
        "5A09558",  # Assistance with Respiratory Ventilation, Greater than 96 Consecutive Hours, Intermittent Positive Airway Pressure
    }

    SNOMEDCT = {
        "1186618000",  # Bilevel artificial ventilation (regime/therapy)
        "1186678006",  # Continuous positive airway pressure with an assured constant airway pressure adjunct (regime/therapy)
        "243142003",  # Dual pressure spontaneous ventilation support (regime/therapy)
        "257420000",  # Breathing circuit (physical object)
        "26397000",  # Breathing bag, device (physical object)
        "297120004",  # Anesthetic face mask (physical object)
        "336602003",  # Oxygen mask (physical object)
        "336623009",  # Oxygen nasal cannula (physical object)
        "34281000175105",  # Nocturnal continuous positive airway pressure (regime/therapy)
        "34291000175108",  # Nocturnal dual pressure spontaneous ventilation support (regime/therapy)
        "405642002",  # Anesthesia breathing circuit (physical object)
        "424172009",  # Dual pressure spontaneous ventilation support weaning protocol (regime/therapy)
        "425478008",  # Blow by oxygen mask (physical object)
        "425826004",  # Bilevel positive airway pressure oxygen nasal cannula (physical object)
        "426294006",  # Face tent oxygen delivery device (physical object)
        "426851007",  # Aerosol oxygen mask (physical object)
        "426854004",  # High flow oxygen nasal cannula (physical object)
        "427591007",  # Nonrebreather oxygen mask (physical object)
        "427594004",  # Oxyhood (physical object)
        "428285009",  # Venturi mask (physical object)
        "442398003",  # Nasal mask (physical object)
        "446573003",  # Continuous positive airway pressure titration (procedure)
        "447243000",  # Bilevel positive airway pressure titration (procedure)
        "448134000",  # Continuous positive airway pressure to nonventilated lung (regime/therapy)
        "463587004",  # Rebreathing oxygen face mask (physical object)
        "464225001",  # Oxygen administration nasal catheter (physical object)
        "464233000",  # Partial-rebreathing oxygen face mask (physical object)
        "464428002",  # Moisture chamber face mask (physical object)
        "464578003",  # Oxygen administration face tent (physical object)
        "465256000",  # Tracheostomy mask, aerosol (physical object)
        "465433006",  # Venturi oxygen face mask (physical object)
        "465839001",  # Tracheostomy mask, oxygen (physical object)
        "466713001",  # Basic nasal oxygen cannula (physical object)
        "467645007",  # Continuous positive airway pressure nasal oxygen cannula (physical object)
        "468246003",  # Aerosol face mask, rebreathing (physical object)
        "468414000",  # Aerosol face mask, non-rebreathing (physical object)
        "469956003",  # Anesthesia breathing circuit circulator (physical object)
        "47545007",  # Continuous positive airway pressure ventilation treatment (regime/therapy)
        "701254008",  # Bidirectional-anesthesia breathing circuit (physical object)
        "701582002",  # Vortex oxygen face mask (physical object)
        "704718009",  # CPAP/BPAP oral mask (physical object)
        "706175007",  # Ventilator breathing circuit (physical object)
        "706226000",  # Continuous positive airway pressure/Bilevel positive airway pressure mask (physical object)
        "719705009",  # Capnography oxygen mask (physical object)
        "722742002",  # Breathing room air (finding)
    }

class EmergencyDepartmentEvaluation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of an assessment performed in the emergency department.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment that was performed in the emergency department.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Emergency Department Evaluation"
    OID = "2.16.840.1.113762.1.4.1111.163"
    DEFINITION_VERSION = "20200305"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "28568-4",  # Physician Emergency department Note
        "34111-5",  # Emergency department Note
        "34878-9",  # Emergency medicine Note
        "51846-4",  # Emergency department Consult note
        "54094-8",  # Emergency department Triage note
        "68552-9",  # Emergency medicine Emergency department Admission evaluation note
        "74211-4",  # Emergency department+Hospital Summary of episode note
        "78249-0",  # Emergency department Admission history and physical note
        "78307-6",  # Emergency department History and physical note
        "78337-3",  # Emergency department Initial evaluation note
        "78341-5",  # Emergency department Transfer summary note
        "80741-2",  # Emergency medicine Emergency department Plan of care note
        "80746-1",  # Emergency department Plan of care note
        "83818-5",  # Attending Emergency department Note
    }

class TobaccoUseScreening(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments to screen for tobacco use.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment for tobacco use, including smoking and smoke-less tobacco products.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Tobacco Use Screening"
    OID = "2.16.840.1.113883.3.526.3.1278"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "39240-7",  # Tobacco use status CPHS
        "68535-4",  # Have you used tobacco in the last 30 days [SAMHSA]
        "68536-2",  # Have you used smokeless tobacco product in the last 30 days [SAMHSA]
        "72166-2",  # Tobacco smoking status
    }

class FallsScreening(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments using a falls screening tool.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment for fall risk.

    **Exclusion Criteria:** Excludes concepts that represent an assessment for falls for children.
    """

    VALUE_SET_NAME = "Falls Screening"
    OID = "2.16.840.1.113883.3.464.1003.118.12.1028"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "52552-7",  # Falls in the past year
        "57254-5",  # Standardized fall risk assessment was conducted during assessment period [CMS Assessment]
        "59454-9",  # History of falling; immediate or within 3 months [Morse Fall Scale]
        "73830-2",  # Fall risk assessment
    }

class StandardizedToolsScoreForAssessmentOfCognition(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to define concepts for assessments representing total score results for standardized tools used for the evaluation of cognition.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that describe assessments with total score results for the following standardized tools: -Blessed Orientation-Memory-Concentration Test (BOMC) -Montreal Cognitive Assessment (MoCA) -St. Louis University Mental Status Examination (SLUMS) -Mini-Mental State Examination (MMSE) [Note: The MMSE has not been well validated for non-Alzheimer's dementias] -Short Informant Questionnaire on Cognitive Decline in the Elderly (IQCODE) -Ascertain Dementia 8 (AD8) Questionnaire -Minimum Data Set (MDS) Brief Interview of Mental Status (BIMS) [Note: Validated for use with nursing home patients only] -Formal neuropsychological evaluation -Mini-Cog.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Standardized Tools Score for Assessment of Cognition"
    OID = "2.16.840.1.113883.3.526.3.1006"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "58151-2",  # Prior assessment Brief Interview for Mental Status (BIMS) summary score [MDSv3]
        "71492-3",  # Total score [SLUMS]
        "71493-1",  # Total score [IQCODE]
        "71722-3",  # Total score [AD8]
        "72106-8",  # Total score [MMSE]
        "72172-0",  # Total score [MoCA]
        "72173-8",  # Total score [BOMC]
        "72233-0",  # Total score [Mini-Cog]
    }

class StandardizedPainAssessmentTool(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments using pain-focused tools or instruments to quantify pain intensity.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment of pain intensity.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Standardized Pain Assessment Tool"
    OID = "2.16.840.1.113883.3.526.3.1028"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "38208-5",  # Pain severity - Reported
        "38214-3",  # Pain severity [Score] Visual analog score
        "38221-8",  # Pain severity Wong-Baker FACES pain rating scale
        "72514-3",  # Pain severity - 0-10 verbal numeric rating [Score] - Reported
        "77565-0",  # Pain interference score [BPI Short Form]
    }

class Phq9AndPhq9mTools(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for the assessments of PHQ 9 and PHQ 9M resulting in a completed depression assessment scores for adults and adolescents.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent a completed assessment using PHQ 9 and PHQ 9M depression assessment tools for adults and adolescents with a summary score.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "PHQ 9 and PHQ 9M Tools"
    OID = "2.16.840.1.113883.3.67.1.101.1.263"
    DEFINITION_VERSION = "20220219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "44261-6",  # Patient Health Questionnaire 9 item (PHQ-9) total score [Reported]
        "89204-2",  # Patient Health Questionnaire-9: Modified for Teens total score [Reported.PHQ.Teen]
    }

class AbnormalPresentation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to group diagnoses of a fetus in an abnormal position in the uterus at the time of delivery.

    **Data Element Scope:** This value set may use a model element related to diagnosis.

    **Inclusion Criteria:** Includes concepts that identify a diagnosis of a fetus in an abnormal position.

    **Exclusion Criteria:** Excludes concepts that represent a fetus in a vertex presentation.
    """

    VALUE_SET_NAME = "Abnormal Presentation"
    OID = "2.16.840.1.113762.1.4.1045.105"
    DEFINITION_VERSION = "20210611"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10CM = {
        "O321XX0",  # Maternal care for breech presentation, not applicable or unspecified
        "O321XX1",  # Maternal care for breech presentation, fetus 1
        "O321XX2",  # Maternal care for breech presentation, fetus 2
        "O321XX3",  # Maternal care for breech presentation, fetus 3
        "O321XX4",  # Maternal care for breech presentation, fetus 4
        "O321XX5",  # Maternal care for breech presentation, fetus 5
        "O321XX9",  # Maternal care for breech presentation, other fetus
        "O322XX0",  # Maternal care for transverse and oblique lie, not applicable or unspecified
        "O322XX1",  # Maternal care for transverse and oblique lie, fetus 1
        "O322XX2",  # Maternal care for transverse and oblique lie, fetus 2
        "O322XX3",  # Maternal care for transverse and oblique lie, fetus 3
        "O322XX4",  # Maternal care for transverse and oblique lie, fetus 4
        "O322XX5",  # Maternal care for transverse and oblique lie, fetus 5
        "O322XX9",  # Maternal care for transverse and oblique lie, other fetus
        "O323XX0",  # Maternal care for face, brow and chin presentation, not applicable or unspecified
        "O323XX1",  # Maternal care for face, brow and chin presentation, fetus 1
        "O323XX2",  # Maternal care for face, brow and chin presentation, fetus 2
        "O323XX3",  # Maternal care for face, brow and chin presentation, fetus 3
        "O323XX4",  # Maternal care for face, brow and chin presentation, fetus 4
        "O323XX5",  # Maternal care for face, brow and chin presentation, fetus 5
        "O323XX9",  # Maternal care for face, brow and chin presentation, other fetus
        "O328XX0",  # Maternal care for other malpresentation of fetus, not applicable or unspecified
        "O328XX1",  # Maternal care for other malpresentation of fetus, fetus 1
        "O328XX2",  # Maternal care for other malpresentation of fetus, fetus 2
        "O328XX3",  # Maternal care for other malpresentation of fetus, fetus 3
        "O328XX4",  # Maternal care for other malpresentation of fetus, fetus 4
        "O328XX5",  # Maternal care for other malpresentation of fetus, fetus 5
        "O328XX9",  # Maternal care for other malpresentation of fetus, other fetus
        "O329XX0",  # Maternal care for malpresentation of fetus, unspecified, not applicable or unspecified
        "O329XX1",  # Maternal care for malpresentation of fetus, unspecified, fetus 1
        "O329XX2",  # Maternal care for malpresentation of fetus, unspecified, fetus 2
        "O329XX3",  # Maternal care for malpresentation of fetus, unspecified, fetus 3
        "O329XX4",  # Maternal care for malpresentation of fetus, unspecified, fetus 4
        "O329XX5",  # Maternal care for malpresentation of fetus, unspecified, fetus 5
        "O329XX9",  # Maternal care for malpresentation of fetus, unspecified, other fetus
        "O632",  # Delayed delivery of second twin, triplet, etc.
        "O641XX0",  # Obstructed labor due to breech presentation, not applicable or unspecified
        "O641XX1",  # Obstructed labor due to breech presentation, fetus 1
        "O641XX2",  # Obstructed labor due to breech presentation, fetus 2
        "O641XX3",  # Obstructed labor due to breech presentation, fetus 3
        "O641XX4",  # Obstructed labor due to breech presentation, fetus 4
        "O641XX5",  # Obstructed labor due to breech presentation, fetus 5
        "O641XX9",  # Obstructed labor due to breech presentation, other fetus
        "O642XX0",  # Obstructed labor due to face presentation, not applicable or unspecified
        "O642XX1",  # Obstructed labor due to face presentation, fetus 1
        "O642XX2",  # Obstructed labor due to face presentation, fetus 2
        "O642XX3",  # Obstructed labor due to face presentation, fetus 3
        "O642XX4",  # Obstructed labor due to face presentation, fetus 4
        "O642XX5",  # Obstructed labor due to face presentation, fetus 5
        "O642XX9",  # Obstructed labor due to face presentation, other fetus
        "O643XX0",  # Obstructed labor due to brow presentation, not applicable or unspecified
        "O643XX1",  # Obstructed labor due to brow presentation, fetus 1
        "O643XX2",  # Obstructed labor due to brow presentation, fetus 2
        "O643XX3",  # Obstructed labor due to brow presentation, fetus 3
        "O643XX4",  # Obstructed labor due to brow presentation, fetus 4
        "O643XX5",  # Obstructed labor due to brow presentation, fetus 5
        "O643XX9",  # Obstructed labor due to brow presentation, other fetus
        "O644XX0",  # Obstructed labor due to shoulder presentation, not applicable or unspecified
        "O644XX1",  # Obstructed labor due to shoulder presentation, fetus 1
        "O644XX2",  # Obstructed labor due to shoulder presentation, fetus 2
        "O644XX3",  # Obstructed labor due to shoulder presentation, fetus 3
        "O644XX4",  # Obstructed labor due to shoulder presentation, fetus 4
        "O644XX5",  # Obstructed labor due to shoulder presentation, fetus 5
        "O644XX9",  # Obstructed labor due to shoulder presentation, other fetus
        "O648XX0",  # Obstructed labor due to other malposition and malpresentation, not applicable or unspecified
        "O648XX1",  # Obstructed labor due to other malposition and malpresentation, fetus 1
        "O648XX2",  # Obstructed labor due to other malposition and malpresentation, fetus 2
        "O648XX3",  # Obstructed labor due to other malposition and malpresentation, fetus 3
        "O648XX4",  # Obstructed labor due to other malposition and malpresentation, fetus 4
        "O648XX5",  # Obstructed labor due to other malposition and malpresentation, fetus 5
        "O648XX9",  # Obstructed labor due to other malposition and malpresentation, other fetus
    }

    SNOMEDCT = {
        "11914001",  # Transverse OR oblique presentation of fetus (disorder)
        "1231436008",  # Breech position of fetus in pregnancy (disorder)
        "18559007",  # Frank breech presentation (finding)
        "199355003",  # Breech presentation with antenatal problem (finding)
        "199359009",  # Oblique lie with antenatal problem (disorder)
        "199363002",  # Transverse lie with antenatal problem (disorder)
        "199751005",  # Obstructed labor due to breech presentation (disorder)
        "199752003",  # Obstructed labor due to face presentation (disorder)
        "199753008",  # Obstructed labor due to brow presentation (disorder)
        "199754002",  # Obstructed labor due to shoulder presentation (disorder)
        "200142000",  # Breech extraction - delivered (finding)
        "21882006",  # Face presentation of fetus (disorder)
        "23954006",  # Acromion presentation (finding)
        "249064003",  # Oblique lie, head in iliac fossa (disorder)
        "249065002",  # Oblique lie, breech in iliac fossa (disorder)
        "249089009",  # Fetal mouth presenting (disorder)
        "249090000",  # Fetal ear presenting (disorder)
        "249091001",  # Fetal nose presenting (disorder)
        "249093003",  # Presentation of orbital ridges of fetus (disorder)
        "249097002",  # Footling breech presentation (finding)
        "249098007",  # Knee presentation (disorder)
        "249099004",  # Single knee presentation (disorder)
        "249100007",  # Double knee presentation (disorder)
        "249104003",  # Dorsoanterior shoulder presentation (disorder)
        "249105002",  # Dorsoposterior shoulder presentation (disorder)
        "274128005",  # Brow delivery (finding)
        "274129002",  # Face delivery (finding)
        "289355004",  # Oblique lie head in right iliac fossa (disorder)
        "289356003",  # Oblique lie head in left iliac fossa (disorder)
        "289357007",  # Oblique lie breech in right iliac fossa (disorder)
        "289358002",  # Oblique lie breech in left iliac fossa (disorder)
        "289398000",  # Fetal breech palpable vaginally (finding)
        "38049006",  # Incomplete breech presentation (finding)
        "48906005",  # Breech presentation, double footling (finding)
        "49168004",  # Complete breech presentation (finding)
        "58903006",  # Breech presentation, single footling (finding)
        "6096002",  # Breech presentation (finding)
        "63750008",  # Oblique lie (disorder)
        "73161006",  # Transverse lie (disorder)
        "79255005",  # Mentum presentation of fetus (disorder)
        "8014007",  # Brow presentation of fetus (disorder)
    }

class HistoryOfAtrialAblation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a situation of a personal history of an atrial ablation.

    **Data Element Scope:** This value set may use a model element related to a diagnosis or assessment.

    **Inclusion Criteria:** Includes concepts that identify a situation for a personal history of an atrial ablation.

    **Exclusion Criteria:** No Exclusions.
    """

    VALUE_SET_NAME = "History of Atrial Ablation"
    OID = "2.16.840.1.113762.1.4.1110.76"
    DEFINITION_VERSION = "20221115"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "427951003",  # History of radiofrequency ablation operation for arrhythmia (situation)
        "429508000",  # History of ablation of atrioventricular node (situation)
        "429756009",  # History of radiofrequency ablation operation on left atrium for arrhythmia (situation)
    }

class MalnutritionRiskScreening(ValueSet):
    """
    **Clinical Focus:** This set of values indicates that a malnutrition screening was done by the health care professional.

    **Data Element Scope:** Data elements included in this set indicate an individual has been screened for malnutrition.

    **Inclusion Criteria:** Malnutrition Risk Screening tools and codes to indicate that a malnutrition risk screening was done.

    **Exclusion Criteria:** Nutrition assessment tools and codes to indicate that a nutrition assessment was done are excluded.
    """

    VALUE_SET_NAME = "Malnutrition Risk Screening"
    OID = "2.16.840.1.113762.1.4.1095.92"
    DEFINITION_VERSION = "20240117"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "101807-6",  # Malnutrition risk screen results indicator
        "98967-3",  # Nutritional Risk Screening 2002 panel
        "98968-1",  # Initial screening NRS_2002
        "98972-3",  # Final screening NRS_2002
    }

class NutritionAssessment(ValueSet):
    """
    **Clinical Focus:** This set of values indicates that a nutrition assessment was completed by a health professional.

    **Data Element Scope:** Completed nutrition assessment by a health professional.

    **Inclusion Criteria:** Nutrition assessment tools and codes to indicate that a nutrition assessment was completed.

    **Exclusion Criteria:** Nutrition screening tools and codes to indicate that a nutrition screening was completed are excluded.
    """

    VALUE_SET_NAME = "Nutrition Assessment"
    OID = "2.16.840.1.113762.1.4.1095.21"
    DEFINITION_VERSION = "20240117"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    LOINC = {
        "101819-1",  # Nutritional assessment for malnutrition status
        "75282-4",  # Nutrition assessment panel
        "75285-7",  # Comparative nutrition assessment standards panel
        "75303-8",  # Nutrition assessment Narrative
        "75304-6",  # Nutrition status observation panel
    }

__exports__ = (
    "EstimatedGestationalAgeAtDelivery",
    "NonInvasiveOxygenTherapy",
    "EmergencyDepartmentEvaluation",
    "TobaccoUseScreening",
    "FallsScreening",
    "StandardizedToolsScoreForAssessmentOfCognition",
    "StandardizedPainAssessmentTool",
    "Phq9AndPhq9mTools",
    "AbnormalPresentation",
    "HistoryOfAtrialAblation",
    "MalnutritionRiskScreening",
    "NutritionAssessment",
)



from ..value_set import ValueSet


class FrailtyDevice(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of devices for durable medical equipment (DME) used by frail patients.

    **Data Element Scope:** This value set may use a model element related to Device.

    **Inclusion Criteria:** Includes concepts that represent  durable medical equipment (DME) or devices for frailty.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS134v10, CMS165v10, CMS131v10, CMS122v10, CMS125v10, CMS130v10
    """

    VALUE_SET_NAME = "Frailty Device"
    OID = "2.16.840.1.113883.3.464.1003.118.12.1300"
    DEFINITION_VERSION = "20200310"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    SNOMEDCT = {
        "23366006",  # Motorized wheelchair device (physical object)
        "23562009",  # Household ventilator, device (physical object)
        "37874008",  # Continuing positive airway pressure unit, device (physical object)
        "58938008",  # Wheelchair device (physical object)
        "66435007",  # Electric bed, device (physical object)
        "71545009",  # Household humidifier, device (physical object)
        "87405001",  # Cane, device (physical object)
        "183240000",  # Self-propelled wheelchair (physical object)
        "183241001",  # Pedal powered wheelchair (physical object)
        "183248007",  # Attendant powered wheelchair (physical object)
        "228869008",  # Manual wheelchair (physical object)
        "261323006",  # Portable oxygen cylinder (physical object)
        "262177002",  # Static oxygen cylinder (physical object)
        "360006004",  # Walking stick (physical object)
        "360008003",  # Commode (physical object)
        "360299009",  # Long cane (physical object)
        "371786002",  # Pressure support ventilator (physical object)
        "391685000",  # Oxygen gas cylinder DD (physical object)
        "391686004",  # Oxygen gas cylinder HD (physical object)
        "391687008",  # Oxygen gas cylinder RD (physical object)
        "391688003",  # Oxygen gas cylinder DF (physical object)
        "391689006",  # Oxygen gas cylinder HX (physical object)
        "391880008",  # Oxygen gas cylinder F (physical object)
        "391881007",  # Oxygen gas cylinder AF (physical object)
        "401953003",  # Oxygen composite cylinder with integral headset 1360 liters (physical object)
        "401954009",  # Oxygen cylinder 1360 liters (physical object)
        "401955005",  # Oxygen cylinder 300 liters (physical object)
        "426160001",  # Oxygen ventilator (physical object)
        "462987000",  # Patient/medical device walker (physical object)
        "463093001",  # Patient/medical device walker, home-use (physical object)
        "464002006",  # Portable ventilator, electric (physical object)
        "464157006",  # Multiple-base walking stick (physical object)
        "464405003",  # Multi-terrain sports wheelchair, attendant/occupant-driven (physical object)
        "464443000",  # Stand-up wheelchair (physical object)
        "464571009",  # Multi-terrain sports wheelchair, electric-motor-driven (physical object)
        "464752005",  # Multi-terrain sports wheelchair, occupant-driven (physical object)
        "465159000",  # Stair-climbing wheelchair (physical object)
        "465556004",  # Single-base walking stick (physical object)
        "465565006",  # Transport wheelchair, collapsible (physical object)
        "465921009",  # Ventilation rocking bed (physical object)
        "466182009",  # Wheelchair, occupant-driven, front-wheels-operated, non-collapsible (physical object)
        "466193006",  # Wheelchair, power-assisted, occupant-controlled, non-collapsible (physical object)
        "466213002",  # Wheelchair, electric-motor-driven, occupant-controlled, manual-steering, collapsible (physical object)
        "466229005",  # Wheelchair, occupant-driven, bimanual-lever-operated, non-collapsible (physical object)
        "466284002",  # Wheelchair, attendant/occupant-driven, bimanual-lever-operated, collapsible (physical object)
        "466316007",  # Wheelchair, combustion-engine-driven, non-collapsible (physical object)
        "466317003",  # Basic walker, foldable (physical object)
        "466322003",  # Wheelchair, power-assisted, attendant/occupant-controlled, non-collapsible (physical object)
        "466331003",  # Wheelchair, attendant/occupant-driven, single-rear-wheel-operated, non-collapsible (physical object)
        "466337004",  # Wheelchair, attendant/occupant-driven, foot-operated, non-collapsible (physical object)
        "466340004",  # Wheelchair, occupant-driven, bimanual-lever-operated, collapsible (physical object)
        "466344008",  # Wheelchair, electric-motor-driven, occupant-controlled, powered-steering, non-collapsible (physical object)
        "466351004",  # Walking chair, foldable (physical object)
        "466364003",  # Wheelchair, attendant/occupant-driven, rear-wheels-operated, collapsible (physical object)
        "466365002",  # Wheelchair, electric-motor-driven, attendant/occupant-controlled, manual-steering, collapsible (physical object)
        "466366001",  # Wheelchair, attendant/occupant-driven, single-lever-operated, non-collapsible (physical object)
        "466378002",  # Wheelchair, electric-motor-driven, occupant-controlled, powered-steering, collapsible (physical object)
        "466381007",  # Wheelchair, attendant-driven, non-collapsible (physical object)
        "466407009",  # Walking stick/seat (physical object)
        "466464004",  # Basic walker, non-foldable (physical object)
        "466466002",  # Wheelchair, occupant-driven, foot-operated, collapsible (physical object)
        "466473007",  # Wheelchair, electric-motor-driven, attendant-controlled, manual-steering, collapsible (physical object)
        "466477008",  # Wheelchair, electric-motor-driven, occupant-controlled, manual-steering, non-collapsible (physical object)
        "466486003",  # Wheelchair, attendant/occupant-driven, bimanual-lever-operated, non-collapsible (physical object)
        "466494005",  # Wheelchair, power-assisted, attendant-controlled, collapsible (physical object)
        "466524001",  # Wheelchair, attendant-driven, collapsible (physical object)
        "466533004",  # Wheelchair, attendant/occupant-driven, foot-operated, collapsible (physical object)
        "466538008",  # Room humidifier (physical object)
        "466550002",  # Wheelchair, attendant/occupant-driven, rear-wheels-operated, non-collapsible (physical object)
        "466553000",  # Wheelchair, occupant-driven, single-lever-operated, collapsible (physical object)
        "466576002",  # Wheelchair, occupant-driven, bimanual-chain-operated, collapsible (physical object)
        "466607002",  # Wheelchair, attendant/occupant-driven, single-lever-operated, collapsible (physical object)
        "466616003",  # Wheelchair, attendant/occupant-driven, single-rear-wheel-operated, collapsible (physical object)
        "466619005",  # Wheelchair, attendant/occupant-driven, bimanual-chain-operated, non-collapsible (physical object)
        "466644002",  # Wheelchair, occupant-driven, bimanual-chain-operated, non-collapsible (physical object)
        "466671002",  # Wheelchair, combustion-engine-driven, collapsible (physical object)
        "466695000",  # Wheelchair, attendant/occupant-driven, single-front-wheel-operated, collapsible (physical object)
        "466699006",  # Wheelchair, power-assisted, attendant/occupant-controlled, collapsible (physical object)
        "466721007",  # Wheelchair, power-assisted, attendant-controlled, non-collapsible (physical object)
        "466739003",  # Wheelchair, occupant-driven, front-wheels-operated, collapsible (physical object)
        "466758007",  # Wheelchair, electric-motor-driven, attendant-controlled, manual-steering, non-collapsible (physical object)
        "466786004",  # Basic electric hospital bed (physical object)
        "466809001",  # Wheelchair, attendant/occupant-driven, bimanual-chain-operated, collapsible (physical object)
        "466813008",  # Wheelchair, electric-motor-driven, attendant-controlled, powered-steering, non-collapsible (physical object)
        "466851008",  # Wheelchair, attendant/occupant-driven, front-wheels-operated, collapsible (physical object)
        "466871004",  # Wheelchair, electric-motor-driven, attendant/occupant-controlled, powered-steering, non-collapsible (physical object)
        "466889003",  # Wheelchair, attendant/occupant-driven, single-front-wheel-operated, non-collapsible (physical object)
        "466926008",  # Wheelchair, occupant-driven, rear-wheels-operated, collapsible (physical object)
        "466927004",  # Wheelchair, electric-motor-driven, attendant/occupant-controlled, powered-steering, collapsible (physical object)
        "466938004",  # Wheelchair, occupant-driven, single-rear-wheel-operated, collapsible (physical object)
        "466947007",  # Wheelchair, occupant-driven, rear-wheels-operated, non-collapsible (physical object)
        "466966007",  # Wheelchair, occupant-driven, foot-operated, non-collapsible (physical object)
        "466986006",  # Walking table (physical object)
        "466989004",  # Wheelchair, attendant/occupant-driven, front-wheels-operated, non-collapsible (physical object)
        "466999009",  # Wheelchair, electric-motor-driven, attendant-controlled, powered-steering, collapsible (physical object)
        "467018005",  # Wheelchair, occupant-driven, single-lever-operated, non-collapsible (physical object)
        "467065004",  # Wheelchair, occupant-driven, single-front-wheel-operated, collapsible (physical object)
        "467077009",  # Wheelchair, electric-motor-driven, attendant/occupant-controlled, manual-steering, non-collapsible (physical object)
        "467095007",  # Wheelchair, power-assisted, occupant-controlled, collapsible (physical object)
        "467137003",  # Wheelchair, occupant-driven, single-rear-wheel-operated, non-collapsible (physical object)
        "467155007",  # Walking chair, non-foldable (physical object)
        "467163008",  # Wheelchair, occupant-driven, single-front-wheel-operated, non-collapsible (physical object)
        "469361005",  # Heat/moisture exchanger, reusable (physical object)
        "469860004",  # All-plastic conventional wheelchair (physical object)
        "470119002",  # Heat/moisture exchanger, single-use (physical object)
        "470174002",  # Heat/moisture exchanger insertable filter (physical object)
        "700593005",  # Heated respiratory humidifier (physical object)
        "700705005",  # Non-heated respiratory humidifier (physical object)
        "700910000",  # Ultrasonic respiratory humidifier (physical object)
        "702172008",  # Home continuous positive airway pressure unit (physical object)
        "702173003",  # Home bilevel positive airway pressure unit (physical object)
        "705404007",  # Walker/Walking frame (physical object)
        "705406009",  # Walker (physical object)
        "705419008",  # Special-function wheelchair (physical object)
        "705421003",  # Sports wheelchair (physical object)
        "705422005",  # Power-driven wheelchair (physical object)
        "705423000",  # Electric-motor-driven wheelchair (physical object)
        "705425007",  # Attendant/occupant-controlled electric-motor-driven wheelchair (physical object)
        "705426008",  # Attendant-controlled electric-motor-driven wheelchair (physical object)
        "705427004",  # Power-assisted wheelchair (physical object)
        "705428009",  # Manual-driven wheelchair (physical object)
        "706180003",  # Respiratory humidifier (physical object)
        "714700001",  # Bilevel positive airway pressure unit hand held (physical object)
        "781471009",  # Gait rehabilitation electronic walker (physical object)
    }


class CardiacPacer(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a cardiac pacer device.

    **Data Element Scope:** This value set may use a model element related to Device.

    **Inclusion Criteria:** Includes concepts that represent a device that is a cardiac pacer.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS144v10, CMS145v10
    """

    VALUE_SET_NAME = "Cardiac Pacer"
    OID = "2.16.840.1.113883.3.526.3.1193"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    SNOMEDCT = {
        "14106009",  # Cardiac pacemaker, device (physical object)
        "56961003",  # Cardiac transvenous pacemaker, device (physical object)
        "360127006",  # Intravenous cardiac pacemaker system (physical object)
        "360128001",  # Intravenous triggered cardiac pacemaker system (physical object)
        "424921004",  # Permanent cardiac pacemaker, device (physical object)
    }


__exports__ = (
    "CardiacPacer",
    "FrailtyDevice",
)



import json
from dataclasses import dataclass
from typing import Any

from canvas_generated.messages.effects_pb2 import Effect
from canvas_sdk.base import TrackableFieldsModel


@dataclass
class Metadata:
    """A class representing a metadata."""

    key: str
    value: str

    def to_dict(self) -> dict[str, Any]:
        """Convert the metadata to a dictionary."""
        return {"key": self.key, "value": self.value}


class BaseMetadata(TrackableFieldsModel):
    """Base class for metadata effects."""

    key: str

    def upsert(self, value: str) -> Effect:
        """Upsert the metadata."""
        self._validate_before_effect("upsert")

        return Effect(
            type=f"UPSERT_{self.Meta.effect_type}",  # type: ignore[attr-defined]
            payload=json.dumps(
                {
                    "data": {**self.values, "value": value},
                }
            ),
        )


__exports__ = ("Metadata",)



from dataclasses import dataclass


@dataclass(frozen=True)
class ParsedEnvelope:
    """Represents the SMTP envelope information from an inbound parsed email."""

    email_from: str
    email_to: list[str]

    @classmethod
    def from_dict(cls, data: dict) -> "ParsedEnvelope":
        """Create ParsedEnvelope instance from dictionary."""
        return cls(
            email_from=data["from"],
            email_to=data["to"],
        )

    def to_dict(self) -> dict:
        """Convert parsed envelope to dictionary format."""
        return {
            "emailFrom": self.email_from,
            "emailTo": self.email_to,
        }


__exports__ = ()
